/*
** bayer2rgbstereo.c
** compile with: make bayer2rgbstereo
*/

#include <math.h>
#include <string.h>
#include "mex.h"
#include "image_buffer.h"
#include "mexutil.h"
#include "bayer.h"

/*
**  mexFunction is the interface function to MATLAB.
**
** plhs - pointer to left-hand OUTPUT mxArrays
** nlhs - number of left-hand OUTPUT arrays
** prhs - pointer to right-hand INPUT mxArrays
** nrhs - number of right-hand INPUT arrays
**
*/
void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
int mdims[]={0,0,0};
int out_dims[]={0,0,0};
buffer *bf_image;
buffer *bf_il,*bf_ir;
mxArray *mx_il,*mx_ir;
unsigned char *field_ptr;
int rows,cols;
int ssflag=1;             /* Default */
int bpattern=BAYER_RGGB;  /* Default */
int fields=2;             /* Default */

/* Argument count check */
 if((nrhs <1)||(nrhs>4)) {
   if(nrhs !=0) mexPrintf("Error: 1-4 input arguments expected.\n");
   goto mexErrExit;
 }
    
 /* Parse arg 1 (sensor array) */
 mdims[0]=-1;mdims[1]=-1;
 if(!type_check("buf",prhs[0],mxUINT8_CLASS,2,mdims))
   goto mexErrExit;
 
 /* Embed image in image buffer struct */
 bf_image = buffer_encapsulate(prhs[0]);
 rows = bf_image->rows;
 cols = bf_image->cols;

 if(nrhs>1) {
   /* Parse arg 2 (ssflag) */
   mdims[0]=1;mdims[1]=1;
   if(!type_check("ssflag",prhs[1],mxDOUBLE_CLASS,2,mdims))
     goto mexErrExit;
   ssflag=mxGetScalar(prhs[1]);
 }

 if(nrhs>2) {
   /* Parse arg 3 (bpattern) */
   mdims[0]=1;mdims[1]=1;
   if(!type_check("bpattern",prhs[2],mxDOUBLE_CLASS,2,mdims))
     goto mexErrExit;
   bpattern=mxGetScalar(prhs[2]);
 }
 /* Clamp bpattern */
 if(bpattern<BAYER_BGGR) bpattern=BAYER_BGGR;
 if(bpattern>BAYER_GBRG) bpattern=BAYER_GBRG;

 if(nrhs>3) {
   /* Parse arg 4 (fields) */
   mdims[0]=1;mdims[1]=1;
   if(!type_check("fields",prhs[3],mxDOUBLE_CLASS,2,mdims))
     goto mexErrExit;
   fields=mxGetScalar(prhs[3]);
 }
 /* Clamp fields */
 if(fields>3) fields=3;
 if(fields<1) fields=1;

 if(ssflag) {
   out_dims[0]=cols/2;  /* Flip dimensions */
   out_dims[1]=rows/2/fields;
 } else {
   out_dims[0]=cols;  /* Flip dimensions */
   out_dims[1]=rows/fields;
 }
 out_dims[2]=3;

 /* Allocate left image */
 if(nlhs>0) {
   mx_il = mxCreateNumericArray(3,out_dims,mxDOUBLE_CLASS,mxREAL);
   bf_il = buffer_encapsulate(mx_il);
   /* Call computation function */
   field_ptr=(unsigned char *)bf_image->data;
   if(ssflag) bayer_to_rgb_fields_ds(field_ptr,bf_il,bpattern,fields,0);
   else       bayer_to_rgb_fields(field_ptr,bf_il,bpattern,fields,0);
 }

 /* Allocate right image */
 if(nlhs>1) {
   mx_ir = mxCreateNumericArray(3,out_dims,mxDOUBLE_CLASS,mxREAL);
   bf_ir = buffer_encapsulate(mx_ir);
   /* Call computation function */
   field_ptr=(unsigned char *)bf_image->data;
   if(ssflag) bayer_to_rgb_fields_ds(field_ptr,bf_ir,bpattern,fields,1);
   else       bayer_to_rgb_fields(field_ptr,bf_ir,bpattern,fields,1);
 }

 if(nlhs>0) plhs[0] = mx_il;
 if(nlhs>1) plhs[1] = mx_ir;

 /* Free memory */
 free(bf_image);  /* Only release struct */
  
 /* Jump to normal exit */
 goto mexExit;

 /* Exit with error message */
mexErrExit:
 mexPrintf("Usage: [<il>,<ir>]=bayer2rgbstereo(<buf>,[<ssflag>,<bpattern>,<fields>])\n");
mexExit: {}
}
