%
% Simple test of new camera IF
% Grab and display using current camera settings
%

subsamplefl=1;   % Set to 0 for hires, 1 for lowres
infofl=1;        % Print camera info if set
bayer_pattern=0; % BGGR  (for BB-HICOL-40)
%bayer_pattern=2; % RGGB (for BB-COL-40)

K=100;    % Number of frames to grab and display

% Connect to camera
camera('connect',infofl);
% Set a white balance
camera('whitebalance',50,33); % Use whitebalance script to adjust

vm=camera('video_mode');      % Ask for current video mode

% Try to set highest possible frame rate
fr=30*[0.125 0.25 0.5 1];
fr=60*[0.0625 0.125 0.25 0.5 1];  % 1394b
for k=1:length(fr)
  frame_rate=camera('framerate',fr(k));
end
fprintf('framerate=%d\n',frame_rate);

% Prepare to grab (allocate ring buffers etc)
camera('setup_capture');

% Check if bayer pattern or not
if(length(strfind(vm,'MONO'))),
  fields=1;
  % Check if stereo or not
  if(length(strfind(vm,'MONO16')))
    fields=2;
    for k=1:K,
      a=camera('getframe');
      % Convert sensor reading to two bitmaps
      [il,ir]=bayer2rgbstereo(a,subsamplefl,bayer_pattern,fields);
    
      figure(1);clf
      image([il ir]);axis image
      ts=sprintf('%d(%d)',k,K);
      title(ts);
      drawnow
    end
  else
    for k=1:K,
      a=camera('getframe');
      % Convert sensor reading to bitmap
      im=bayer2rgbstereo(a,subsamplefl,bayer_pattern,fields);
    
      figure(1);clf
      image(im);axis image
      ts=sprintf('%d(%d)',k,K);
      title(ts);
      drawnow
    end    
  end  
else
  fmt=vm(strfind(vm,'_')+1:end);
  switch(fmt)
   case 'RGB', buf_type=1;
   case 'YUV444', buf_type=2;
   case 'YUV422', buf_type=3;
   case 'YUV411', buf_type=4;
  end

  for k=1:K,
    a=camera('getframe');
    im=dcam2rgb(a,buf_type); % Convert sensor reading to bitmap
    
    figure(1);clf
    image(im);axis image
    ts=sprintf('%d(%d)',k,K);
    title(ts);
    drawnow
  end
end

% Release buffers
camera('stop_capture');

% Release camera
camera('disconnect');