/*
** dcam2rgb.c
** compile with: make dcam2rgb
*/

#include <math.h>
#include <string.h>
#include "mex.h"
#include "image_buffer.h"
#include "mexutil.h"
#include "dcam.h"

/*
**  mexFunction is the interface function to MATLAB.
**
** plhs - pointer to left-hand OUTPUT mxArrays
** nlhs - number of left-hand OUTPUT arrays
** prhs - pointer to right-hand INPUT mxArrays
** nrhs - number of right-hand INPUT arrays
**
*/
void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
int mdims[]={0,0,0};
int out_dims[]={0,0,0};
buffer *bf_image;
buffer *bf_image2;
unsigned char *field_ptr;
int rows,cols;
int buf_type=1;         /* Default */

/* Argument count check */
 if((nrhs <1)||(nrhs>2)) {
   if(nrhs !=0) mexPrintf("Error: 1-2 input arguments expected.\n");
   goto mexErrExit;
 }
    
 /* Parse arg 1 (sensor array) */
 mdims[0]=-1;mdims[1]=-1;
 if(!type_check("buf",prhs[0],mxUINT8_CLASS,2,mdims))
   goto mexErrExit;
 
 /* Embed image in image buffer struct */
 bf_image = buffer_encapsulate(prhs[0]);
 rows = bf_image->rows;
 cols = bf_image->cols;

 if(nrhs>1) {
   /* Parse arg 2 (type) */
   mdims[0]=1;mdims[1]=1;
   if(!type_check("type",prhs[1],mxDOUBLE_CLASS,2,mdims))
     goto mexErrExit;
   buf_type=mxGetScalar(prhs[1]);
 }

 switch(buf_type) {
 case DCAM_RGB:
   out_dims[0]=cols;  /* Flipped dimensions */
   out_dims[1]=rows/3;
   out_dims[2]=3;
   break;
 case DCAM_YUV444:
   out_dims[0]=cols; /* Flipped dimensions */
   out_dims[1]=rows/3;
   out_dims[2]=3;
   break;
 case DCAM_YUV422:
   out_dims[0]=cols; /* Flipped dimensions */
   out_dims[1]=rows/2;
   out_dims[2]=3;
   break;
 case DCAM_YUV411:
   out_dims[0]=cols; /* Flipped dimensions */
   out_dims[1]=(rows*2)/3;
   out_dims[2]=3;
   break;
 default:
   mexPrintf("Buffer type %d not supported\n",buf_type);
   goto mexExit;
 }
 /* Allocate output image */
 if(nlhs>0) {
   plhs[0] = mxCreateNumericArray(3,out_dims,mxDOUBLE_CLASS,mxREAL);
   bf_image2 = buffer_encapsulate(plhs[0]);
   /* Call computation function */
   field_ptr=(unsigned char *)bf_image->data;
   switch(buf_type) {
   case DCAM_RGB:
     dcam_rgb_to_matlab(field_ptr,bf_image2);
     break;
   case DCAM_YUV444:
     dcam_yuv444_to_matlab(field_ptr,bf_image2);
     break;
   case DCAM_YUV422:
     dcam_yuv422_to_matlab(field_ptr,bf_image2);
     break;
   case DCAM_YUV411:
     dcam_yuv411_to_matlab(field_ptr,bf_image2);
     break;
   }
 }

 /* Free memory */
 free(bf_image);  /* Only release struct */
  
 /* Jump to normal exit */
 goto mexExit;

 /* Exit with error message */
mexErrExit:
 mexPrintf("Usage: <im>=dcam2rgb(<buf>[,<type>])\n");
mexExit: {}
}
