/*
  Author: Sancho McCann
  Year: 2007
  
  I, the copyright holder of this work, hereby release it into the
  public domain. This applies worldwide.

  I grant anyone the right to use this work for any purpose, without
  any conditions, unless such conditions are required by law.
*/


#include <boost/filesystem.hpp>
#include <boost/program_options.hpp>
#include <boost/algorithm/string.hpp>
#include <iostream>
#include <string>
#include <set>

namespace po = boost::program_options;
namespace fs = boost::filesystem;
using namespace std;

void extractDescriptorsFromFile(const fs::path & imagePath, const string & exePath);
void extractDescriptorsFromDirectory(const fs::path & directoryPath, const string & exePath);

set<string> validExtensions;

int main(int argc, char** argv)
{
  string inputPath;
  string exePath;
  
  // From OpenCv documentation: these are the
  // file types that cvLoadImage can handle
  validExtensions.insert(".bmp");
  validExtensions.insert(".dib");
  validExtensions.insert(".jpeg");
  validExtensions.insert(".jpg");
  validExtensions.insert(".jpe");
  validExtensions.insert(".png");
  validExtensions.insert(".pbm");
  validExtensions.insert(".pgm");
  validExtensions.insert(".ppm");
  validExtensions.insert(".sr");
  validExtensions.insert(".ras");
  validExtensions.insert(".tiff");
  validExtensions.insert(".tif");

  po::options_description command_line_options("Command line options");
  command_line_options.add_options()
    ("help,H", "produce help message")
    ("exe,E", po::value<string>(), "path to executable")
    ("input,I", po::value<string>(), "input directory")
    ;
  po::positional_options_description positional;
  positional.add("input", -1);
  
  try {
    po::variables_map vm;
    po::store(po::command_line_parser(argc, argv).options(command_line_options).positional(positional).run(),
	      vm);
    po::notify(vm);
    
    if ( vm.count("help") || !vm.count("input") || !vm.count("exe") )
      {
	cout << command_line_options << endl;
	exit(1);
      }
    else
      {
	inputPath = vm["input"].as<string>();
	exePath = vm["exe"].as<string>();
      }
  } catch ( po::invalid_command_line_syntax ) {
    cout << command_line_options << endl;
    exit(1);
  }

  if ( !fs::exists(inputPath) )
    {
      cout << "Directory does not exist." << endl;
      exit(1);
    }
  else if ( !fs::is_directory(inputPath) )
    {
      cout << "Input was not a directory." << endl;
      exit(1);
    }
  else
    {
      extractDescriptorsFromDirectory(inputPath, exePath);
    }
    
  return 0;
}

void extractDescriptorsFromFile(const fs::path & imagePath, const string & exePath)
{
  string command = exePath + " " + imagePath.string() + " " + fs::change_extension( imagePath, ".sift" ).string() + " 3000";
  system(command.c_str());
}

void extractDescriptorsFromDirectory(const fs::path & directoryPath, const string & exePath)
{
  fs::directory_iterator fileIterator(directoryPath);
  fs::directory_iterator end;

  while ( fileIterator != end )
    {
      if ( !fs::is_directory(*fileIterator) &&
	   validExtensions.find( boost::algorithm::to_lower_copy(fs::extension(*fileIterator)) ) != validExtensions.end() )
	{
	  extractDescriptorsFromFile(*fileIterator, exePath);
	}
      ++fileIterator;
    }
}
