%This is the main driver for the image matching
%In ranked order, goes through each class and determines the match quality between the training image and the test image
%results(c,i,j) is a structure that contains diagnostic info for training class c, ranking training image j, and test image i
%results(c,i,j).err is the error
%bestResults(c) is the structure that stores information about the best match for class c
%bestResults(c).bestError is the error
%bestResults(c).bestTest is the test image
%bestResults(c).bestTrain is the train image that matched the test image
%bestResults(c).bestTrans is the best transformation found between the matched the test and train image
%bestResults(c).im is the test image with the bounding box drawn. This is done by applying bestTrans to a bounding box around all of the features of the training image, and then drawing this on the test image (well, actually a box around this transformed box since it may no longer be a box). 
function [results,bestResults] = findMatchingImages(testList,allTrainList,maxRes,imageFlag)
	res=[];
	tic
	bestError=0;
	bestTrans=0;
	%result = zeros(length(testList),length(trainList));
	if nargin<4
		imageFlag = 0;
	end
	if length(maxRes)==1
		maxRes(2) = maxRes;
	end

	bbim=0;
	tic
	testFeatures = loadFeatures(testList,imageFlag,maxRes(2))
	%trainFeatures = loadFeatures(trainList,imageFlag,maxRes(1))
	
	for j=1:length(allTrainList(1).list)
		
		for c = 1:length(allTrainList)
			if j==1
				bestResults(c).bestError = 0;
				bestResults(c).unnormerr=0;
				bestResults(c).done=0;
			end
			goodImages = allTrainList(c).goodImages;
			if j<=length(goodImages) & bestResults(c).done==0
			
				trainFeatures = loadFeatures(allTrainList(c).list(goodImages(j)),0,maxRes(1),1);
				trainList = allTrainList(c).list(goodImages(j));
				img =[c j trainList.num toc]
				for i=1:length(testList)
					if size(trainFeatures.f,2)>0 & size(testFeatures(i).f,2) >0
						[initError,trans,errors,unnormerr] = imageMatch(trainFeatures.f,testFeatures(i).f,trainFeatures.l,testFeatures(i).l);
						results(c,i,j).res =  [testList(i).num trainList.num initError];
						results(c,i,j).err = initError;
						results(c,i,j).trans = trans;
						results(c,i,j).allerr = errors;
						results(c,i,j).unnormerr = unnormerr;
						%if initError>bestResults(c).bestError & (initError-bestResults(c).bestError>0.01 | unnormerr>bestResults(c).unnormerr)
                        if initError>bestResults(c).bestError %& (initError-bestResults(c).bestError>0.01 | unnormerr>bestResults(c).unnormerr)
								
							bestResults(c).bestError=initError;
							bestResults(c).bestTrans = trans;
							bestResults(c).bestTest = i;
							bestResults(c).bestTrain = j;
							bestResults(c).bestRealTrain = trainList.num;
							bestResults(c).unnormerr = unnormerr;
							bestResults(c).allerr = errors;
							%if initError>0.01 
								
								im2size = [max(trainFeatures.l(1,:)) - min(trainFeatures.l(1,:));max(trainFeatures.l(2,:)) - min(trainFeatures.l(2,:))]
								bestResults(c).im = createBBImage(testFeatures(i).im,im2size,trans);
								
								if (initError>0.1)
								%	createFig(testFeatures(i).im,im2size,trans,c);
									bestResults(c).done=1;
									break;
								end
							%end
						end
						
					end
					if toc >25*60
						return
					end
				end
			end
		end
	end

	

function im = createBBImage(im1,im2size,trans)
	
	
		
		points = [1 1;1 im2size(2);im2size(1) 1;im2size(1) im2size(2)]';
		R = trans(1)*[cos(trans(2)) -sin(trans(2));sin(trans(2)) cos(trans(2))];
		p = R*points;
		p=p';
		p(:,1) = p(:,1)-trans(3);
		p(:,2) = p(:,2)-trans(4);
		BoundingBox = [min(p(:,1)) max(p(:,1));min(p(:,2)) max(p(:,2))]'
		im = drawBox(im1,BoundingBox);
		%featmatchim = visualizeCoverage(im,bestl2,besterrors);

function im = createFig(im1,im2size,trans,c)

	
	
		points = [1 1;1 im2size(2);im2size(1) 1;im2size(1) im2size(2)]'
		R = trans(1)*[cos(trans(2)) -sin(trans(2));sin(trans(2)) cos(trans(2))];
		p = R*points;
		p=p';
		p(:,1) = p(:,1)-trans(3);
		p(:,2) = p(:,2)-trans(4);
		figure
		
		imshow(im1);
		title(c)
		line([p(1,2) p(2,2);p(1,2) p(3,2);p(2,2) p(4,2);p(3,2) p(4,2)]', [p(1,1) p(2,1);p(1,1) p(3,1);p(2,1) p(4,1);p(3,1) p(4,1)]');
	
	
function data = loadFeatures(fileList,loadImageFlag,maxRes,shiftmin)
	for i=1:length(fileList)
		[f,l] = loadSift(fileList(i).siftName);
		data(i).l = l;
		data(i).f =f;

		if nargin>3
			data(i).l(1,:) = l(1,:)-min(l(1,:));
			data(i).l(2,:) = l(2,:)-min(l(2,:));
		end
		if loadImageFlag==1
			im = imread(fileList(i).imgName);
			if max(size(im))>maxRes
				data(i).im = imresize(im,maxRes/max(size(im)));
			else
				data(i).im = im;
			end
		end
		
	end

function im = drawBox(im,BB) 
	bs = diff(BB);
	BB(1) = BB(1)-0.1*bs(1);
	BB(2) = BB(2)+0.1*bs(1);
	BB(3) = BB(3)-0.1*bs(2);
	BB(4) = BB(4)+0.1*bs(2);
	BB = round(BB);

	BB(BB<1) = 1;
	
	if BB(2,1)>size(im,1)
		BB(2,1) = size(im,1);
	end
	if BB(2,2)>size(im,2)
		BB(2,2) =size(im,2);
	end
	nr = BB(2,1)-BB(1,1)+1;
	nc = BB(2,2) - BB(1,2)+1;
	if BB(2,1)>size(im,1)
		BB(2,1) = size(im,1)
	end
	try
		index = sub2ind([size(im,1) size(im,2)],[BB(1,1):BB(2,1) BB(1,1):BB(2,1) BB(1,1)*ones(1,nc) BB(2,1)*ones(1,nc)], [BB(1,2)*ones(1,nr) BB(2,2)*ones(1,nr) BB(1,2):BB(2,2) BB(1,2):BB(2,2)]);
	catch
		index = [];
	end
	imsize = size(im,1)*size(im,2);
	
	im(index) = mod(im(index)+128,255);
	im(index+imsize) = mod(im(index+imsize)+128,255);
	im(index+2*imsize) = mod(im(index+2*imsize)+128,255);

