%This function determines the image files in the directory datapath, and possibly compute SIFT descriptors for these images.
%INPUT
%datapath - the directory path we process
%outputpath - the directory that any sift descriptors are stored in
%maxRes - the maximum dimension of an image, it resized otherwise
%imgType - a string such as '.jpg' or '.png';
%numericflag==0 means that all image files are numbers and we sort according to number
%nosiftflag==0 means we compute sift features for the images, other wise we don't
%OUTPUT -
%fileList is an array of structures, each is information for a single image
%fileList(i).imgName = the full path name for ith image in datapath
%fileList(i).siftName = the full path name for sift image in datapath
%fileList(i).valid ===1 means this file is valid, ie. image exists or keypoint extraction worked (i.e don't use this file later if not)
%fileList(i).num is the number if the filename is a number
function fileList = writeSiftImages(datapath,outputpath,maxRes,imgType,numericflag,nosiftflag)
	if nargin<3
		maxRes = 1000;
	end
	if nargin<4
		imgType = '.jpg';
	end
	fileList=[];
	if nargin<5
		numericflag=0;
	end
	if nargin<6
		nosiftflag=0;
	end
	imageList = getImageFileList(datapath,imgType,numericflag);
	c=1;
	for i=1:length(imageList)
		fileList(c).imgName = strcat(datapath,imageList(i).name);
		fileList(c).siftName = strcat(outputpath,imageList(i).name(1:end-length(imgType)),'.sift');
		if nosiftflag==0
			[fileList(c).time,fileList(c).nf, fileList(c).valid] = writeSiftImage(fileList(c).imgName,fileList(c).siftName,maxRes);
			if fileList(c).valid==1
				c=c+1;
			end
		end	
		if numericflag==0
			fileList(c).num = imageList(i).num;
			[isvalid,junk] = system(strcat(['stat ' fileList(c).siftName]));
			if isvalid ~=0
				fileList(c).valid = 0;
			else
				fileList(c).valid = 1;
				c=c+1;
			end
		end
	end
	fileList = fileList(1:c-1);
	
%imageList(i).name - name of the i image of imgType in  directory
%imaeList(i).num - if imageList(i).name is a number, then it's numeric represenation
%flag==0 means we assumed image files are numerically named
function imageList = getImageFileList(directory,imgType,flag)
	fileList = dir(directory);
	nFiles = 1;
	imageList=[];
	isok=1;
	for i=1:length(fileList)
		if fileList(i).isdir==0 & length(strfind(fileList(i).name,imgType))>0
			if flag==0
				[num,isok] = str2num(fileList(i).name(1:end-length(imgType)));
			end
			if flag==1 | isok~=0
				imageList(nFiles).name = fileList(i).name;
				if flag==0
					imageList(nFiles).num = str2num(fileList(i).name(1:end-length(imgType)));
				end
				nFiles = nFiles+1;
			end
		end
	end

	if flag==0 & length(imageList)>0
		[value,index] = sort([imageList.num]);
		imageList = imageList(index);
	end
	
	

function [time,fsize,isvalid] = writeSiftImage(datapath,outputpath,maxRes)
	im  = imread(datapath);
	if max(size(im))>maxRes
		im = imresize(im,maxRes/max(size(im)));
		imwrite(im,datapath);
	end

	thecommand = strcat(['./keypoints ' datapath ' '  outputpath]);

	[a,b]= system(thecommand);
	thecommand
	if a==0
		isvalid = 1;
	else 	
		isvalid=0;
	end
	time=0;
	fsize=0;
	


