/*
  Author: Sancho McCann
  Year: 2007
  
  I, the copyright holder of this work, hereby release it into the
  public domain. This applies worldwide.

  I grant anyone the right to use this work for any purpose, without
  any conditions, unless such conditions are required by law.
*/

#include <boost/filesystem.hpp>
#include <boost/program_options.hpp>
#include <string>
#include <iostream>
#include <sys/wait.h>

namespace po = boost::program_options;
namespace fs = boost::filesystem;
using namespace std;

const int sleeptime = 500000; // half a second

bool getFilesInDirectory(const fs::path & directory, vector<string> & existingFiles);

int main(int argc, char** argv)
{
  po::options_description desc("Options");
  desc.add_options()
    ("directory", po::value<string>()->default_value("./"), "the directory to watch")
    ("exe", po::value<string>(), "the path to the keypoint extraction executable")
    ;

  po::variables_map vm;
  po::store(po::parse_command_line(argc, argv, desc), vm);
  po::notify(vm);
  fs::path watchDirectory(vm["directory"].as<string>());
  
  if ( !vm.count("exe") )
    {
      cout << "No executable given" << endl;
      exit(1);
    }
  string pathToExecutable = vm["exe"].as<string>();

  if ( !fs::exists(watchDirectory) )
    {
      cout << "Given directory does not exist" << endl;
      exit(1);
    }
  if ( !fs::is_directory(watchDirectory) )
    {
      cout << "Given file is not a directory" << endl;
      exit(1);
    }

  vector<string> existingFiles;
  
  while (true)
    {
      if ( getFilesInDirectory(watchDirectory, existingFiles) )
	{
	  string addedFile = existingFiles.back();

	  string command = pathToExecutable + " " + addedFile +
	    " " + watchDirectory.string() + "/" + fs::basename(addedFile)+".sift " + "3000";
	  system(command.c_str());
	}
      else
	{
	  usleep(sleeptime);
	}
    }

  return 0;
}

bool getFilesInDirectory(const fs::path & directory, vector<string> & existingFiles)
{
  fs::directory_iterator directoryIterator(directory);
  fs::directory_iterator end;
  bool newFile = false;

  while ( !newFile && directoryIterator != end )
    {
      if ( std::find(existingFiles.begin(), existingFiles.end(), directoryIterator->string()) == existingFiles.end() )
	{
	  existingFiles.push_back(directoryIterator->string());
	  if ( fs::extension(directoryIterator->string()) == ".jpg" ||
	       fs::extension(directoryIterator->string()) == ".JPG" ||
	       fs::extension(directoryIterator->string()) == ".png" ||
	       fs::extension(directoryIterator->string()) == ".jpeg" ||
	       fs::extension(directoryIterator->string()) == ".JPEG" ||
	       fs::extension(directoryIterator->string()) == ".PNG" )
	    {
	      newFile = true;
	    }
	}
      ++directoryIterator;
    }
      
  return newFile;
}
