%function state=find_saccade(lim,stateo,kpl,kpr,memory,pars,wc)
%
% LIM       Limits for pan and tilt [pm px tm tx]
% KPL       Left pos
% KPR       Right pos
% MEMORY    Struct for memory with fields Ap,Up,Wa,Wu
% PARS      Parameters for PREDICTIVE_MAP
% WC        Image centre [yc xc]
%
%Per-Erik Forssen, Jan 2007
function state=find_saccade(lim,stateo,kpl,kpr,memory,pars,wc)

Ap=memory.Ap;
Up=memory.Up;
Wa=memory.Wa;
Wu=memory.Wu;

%
% Search for state that minimises centering error
%
[x1,x2]=meshgrid([-1 0 1],[-1 0 1]);
x1=x1(:);
x2=x2(:);

% W*p=y => p=Wp*y
W=0.5*[ones(9,1) 2*x1 2*x2 -x1.^2 -x2.^2 -2*x1.*x2];
Wp=pinv(W);

sc=30;        % Current search window
x0=[0;0];     % Current guess
x0(1)=max(lim(1)+sc,min(lim(2)-sc,stateo(1)));
x0(2)=max(lim(3)+sc,min(lim(4)-sc,stateo(2)));
Ni=12;        % Number of search iterations

dx0=30; % To enter loop

% Fill in known values in <a>
a=[kpl(1:2); kpr(1:2);stateo;0;0];
for k=1:Ni,
  Xlc=[x1*sc+x0(1) x2*sc+x0(2)]';
  Ylc=zeros(1,9);
  for l=1:9,
    % Evaluate function in 9 points
    a(7:8)=Xlc(:,l);
    [u_pred,cl]=predictive_map(Ap,Up,Wa,Wu,a,pars);
    Ylc(l)=norm(u_pred(1:2)+u_pred(3:4)-2*wc);
  end
  p=Wp*Ylc';
  
  %End search if singular
  if(p(4)*p(5)-p(6)*p(6)==0) break;end
  C=1/(p(4)*p(5)-p(6)*p(6))*[p(5) -p(6);-p(6) p(4)];
  offset=C*p(2:3);
  dx=sqrt(offset'*offset)*sc;
  %fprintf('dx/dx0=%g\n',dx/dx0);
  if dx>.5*dx0,
    %disp('Warning:Might diverge');
    sc=sc*1.1;  % Increase search range
    dx0=dx*2;
  else    
    x0=x0+offset*sc;
    sc=sc*2/3;
    dx0=dx; % Remember last step length
  end
  % Make sure bounds are inside the possible
  %x0(1)=max(lim(1)+sc,min(lim(2)-sc,x0(1)));
  %x0(2)=max(lim(3)+sc,min(lim(4)-sc,x0(2)));
end
if((x0(1)>lim(1))&(x0(1)<lim(2))&(x0(2)>lim(3))&(x0(2)<lim(4)))
  state=x0;
  fprintf('SEARCH CONVERGED: New pan-tilt state=[%g %g] from map.\n',state);
  no_valid_gaze=0;
else
  fprintf('SEARCH DIVERGED\n');
  state=[];
end
  