%
% Demo of Saccadic Gaze map
% Version that makes use of dense stereo from ptgrey Triclops lib.
%
%Per-Erik Forssen, May 2007

addpath ../stereoCode/
addpath ../matlab_powerbot/
addpath ../../resources/PEImageGrabbing/

configFile = '../stereoCode/bb_id5020143.config';

% Define centre marker
xx=[-1 1 0 0 0];
yy=[0 0 0 -1 1];
msz=20;

powerbot('connect','bart.ubcrobots.net',7001);

lim=powerbot('getlim');
res=powerbot('getres');

lim(1:2)=[-90 90];  % Constrain range

state=powerbot('getpos');

% Grab an image to determine which camera we're using
img=[];
while(length(img)==0) img=powerbot('getframe');end
if(size(img,2)==768)
  bpattern=0;     % BB-HICOL
else
  bpattern=2;     % BB-COL
end

wc=[size(img,2)/2; size(img,1)/4]; % Image centre

disp_res=[240;320];          % Size of disparity image
disp_sc=2*wc(1)/disp_res(1); % Scaling from disp to image

% Load predictive map and define memory struct
load gaze_proto_hires338.mat  % 16000 prototypes from 338 saccades
memory.Ap=Ap;
memory.Up=Up;
memory.Wa=Wa;
memory.Wu=Wu;

% Set parameters for predictive map
pars.itype = 4;
pars.Nn    = 40;
pars.gstd  = 1;

figure(1);clf

% Remember state from previous frame
stateo=state;

loopfl=1;

while(loopfl)
  
  state=powerbot('setpos',state(1),state(2));

  img=[];
  while(length(img)==0) img=powerbot('getframe');end
  [il,ir]=bayer2rgbstereo(img,0,2,2);
  %il=rgb2gray(il);ir=rgb2gray(ir);
  il=il(:,:,2);ir=ir(:,:,2); % Use only green channel
  % Make a stereo composite image to look at
  im=reshape(uint8([il ir zeros(size(ir))]*255),[size(il) 3]);
  % Do stereo and set invalid pixels to zero
  disp=triclops_stereo(im,configFile);
  dmask=double(disp<252);
  disp=double(disp).*dmask;
  subplot(2,2,1);hold off;image(disp*4);axis image;hold on
  colormap(jet)
  title('Current disparity');
  
  subplot(2,2,2);hold off;image(im);axis image;hold on;
  h=plot(xx*msz+wc(2),yy*msz+wc(1),'r');
  set(h,'LineWidth',3.0);
  title(['Current stereo [pan=' num2str(state(1)) ' tilt=' ...
         num2str(state(2)) ']']);

  % Remember last frame
  stateo=state;
  ilo=il;
  iro=ir;

  no_valid_gaze=1;   % Not yet
  while(no_valid_gaze)
    % Get user input
    subplot(2,2,1)
    [x,y,b]=ginput(1);

    if(b>1)
      drawnow;loopfl=0;
      no_valid_gaze=0;
    end
    
    if(loopfl)
      % Extract disparity
      y=round(y);x=round(x);
      d=0;
      if((x>0)&(x<disp_res(2))&(y>0)&(y<disp_res(1)))
        d=disp(round(y),round(x));
      end

      if(d>0) 
        % Compute points in left and right images
        % Assuming no distortion in training set
        % Offsets x position by 32
        pr=[y;x]*disp_sc+[0;32];
        pl=pr-[0;disp_sc*d];
        kpl=[pl;0;0];
        kpr=[pr;0;0];
      
        % Call min value search
        state=find_saccade(lim,stateo,kpl,kpr,memory,pars,wc);
      else
        state=[]; % Just to get to click again
      end
      if(prod(size(state)))
        no_valid_gaze=0;
      else
        fprintf('Please click again.\n');
      end
    end
  end
  
  if(loopfl)
    % Show target in previous frame
    subplot(2,2,3);hold off
    image(repmat(ilo,[1 1 3]));axis image;hold on
    h=plot(xx*msz+pl(2),yy*msz+pl(1),'r');
    set(h,'LineWidth',3.0);
    title('Previous left');
    subplot(2,2,4);hold off
    image(repmat(iro,[1 1 3]));axis image;hold on
    title('Previous right');
    h=plot(xx*msz+pr(2),yy*msz+pr(1),'r');
    set(h,'LineWidth',3.0);
    drawnow;

    % Clamp state to bounds
    state(1)=max(lim(1),min(lim(2),state(1)));
    state(2)=max(lim(3),min(lim(4),state(2)));
  end
end

powerbot('disconnect')
