%function [u_pred,cl]=predictive_map(Ap,Up,Wa,Wu,a,pars)
%
% A      Query vector
% ITYPE  Interpolation type
%
% Returns:
% U_PRED  Predicted positions
% CL      Indices of used prototypes
%
%Per-Erik Forssen, Sept 2006
function [u_pred,cl]=predictive_map(Ap,Up,Wa,Wu,a,pars)

% Extract parameters
itype = pars.itype;
Nn    = pars.Nn;
gstd  = pars.gstd;

%[cl,d2]=fullsearch_neighbours(Ap,Wa*a,Nn);
[cl,d2]=fullsearch_neighboursmex(Ap,Wa*a,Nn);

switch(itype)
 case 0,
  % Constrant model (weighted average)
  w=exp(-.5*d2/gstd^2)'+eps;
  u_pred=inv(Wu)*Up(:,cl)*w/sum(w);
 case 1,
  % Pure linear model
  C=Up(:,cl)*pinv(Ap(:,cl));
  u_pred=inv(Wu)*C*Wa*a;
 case 2,
  % Weighted linear model Up*Wd=C*Ap*Wd
  w=exp(-.5*d2/gstd^2)'+eps;
  Wd=diag(w);
  C=Up(:,cl)*Wd*pinv(Ap(:,cl)*Wd);
  u_pred=inv(Wu)*C*Wa*a;
 case 3,
  % Weighted projective linear model Up*Wd=C*[Ap;1]*Wd
  w=exp(-.5*d2/gstd^2)'+eps;
  Wd=diag(w);
  C=Up(:,cl)*Wd*pinv([Ap(:,cl);ones(1,Nn)]*Wd);
  Wax=diag([diag(Wa);1]);
  u_pred=inv(Wu)*C*Wax*[a;1];
 case 4,
  % Weighted linear model Up*Wd=C*Ap*Wd with local sigma
  %gstd=gstd*mean(d2(1:ceil(Nn/2)));
  gstd=gstd*mean(sqrt(d2(1:ceil(Nn/2))));
  w=exp(-.5*d2/gstd^2)'+eps;
  Wd=diag(w);
  C=Up(:,cl)*Wd*pinv(Ap(:,cl)*Wd);
  u_pred=inv(Wu)*C*Wa*a;  
end

