%
% This script controls the high level behaviours of the robot during the
% image collection phase of the SRVC
%
% David Meger: May 2, 2007
%
%

% Initialization section...

killall;
pause(1);
clear
%clear all;
%close all;
clc;

addpath(genpath('..'));
addpath(genpath('../../resources'));

% Start the timer.
tic;

configPars.usingLaptop=1; % Are we running on the laptop?

% Parameters for spectral saliency
configPars.ss.Kmax=7;         % Pick no more than K salient regions
configPars.ss.scale=4;        % Go three octaves down
configPars.ss.overlap=.5;     % Remove small regions if 50% overlap
configPars.ss.depthoverlap=.15;% Require 15% overlap with objectMask
configPars.ss.minthr=2.0;     % Require min within region to be
                              % above mean(intensity) times this
                              
configPars.runTime = 8;%30;
configPars.stepSize = 3;
configPars.stepTime = 3;
configPars.tableThreshold = 0.5;
                              
if configPars.usingLaptop
  configPars.output_path='/home/lciuser/DRI7/robotImages/';
  configPars.output_path2='/home/lciuser/DRI7/robotImages_bbox/';
  
  configPars.keypointPath='../../../CLASSIFY/Keypoints/keypoints';

  % Start ImageObserver
  system(sprintf('../../../CLASSIFY/ImageObserver/ImageObserver --exe=%s/keypoints --directory=%s &',configPars.keypointPath,configPars.output_path2(1:end-1)));
  
else
  configPars.output_path='~/robotImages/';
  configPars.output_path2='~/robotImages_bbox/';
  %configPars.output_path='/lci/project/raid1/srvc/DRI1/robotImages/';
  %configPars.output_path2='/lci/project/raid1/srvc/DRI1/robotImages_bbox/';
end;

configPars.bpattern=0;  % For BB-HICOL-40
configPars.position_offset = 0.2; % position offset between camera and odometry
configPars.tilt_offset = 1; % tilt offset of 1 degree, determined from PTU calibration
configPars.numPrevious=5; % The number of previous objects to look at

configPars.nTotalPics = uint8(0); % Total number of pictures we've taken
objFilePath1 = sprintf('%sobjects.txt',configPars.output_path);
objFilePath2 = sprintf('%sobjects.txt',configPars.output_path2);
configPars.objectFile1 = fopen(objFilePath1, 'wt');
configPars.objectFile2 = fopen(objFilePath2, 'wt');

configPars.DISPLAY = 1;

% Camera Parameters
cameraPars.configFile = '../stereoCode/bb_id5020143.config';

cameraPars.canonfl=1;   % Use Canon camera if set, otherwise use Bumblebee

if cameraPars.canonfl
  cameraPars.MAX_LOOK_DIST = 5;
  cameraPars.gazeCenter = [326;512];   % Offset origin (y,x) for Canon camera
else
  cameraPars.MAX_LOOK_DIST = 3; % Only look if object is within 3m of camera
  cameraPars.gazeCenter = [384;512];   % Image centre (y,x) (for 1024x768 image)
end;

cameraPars.BB_HEIGHT35 = 1.44; % Height of Bumblebee Camera
cameraPars.BB_HEIGHT40 = 1.41; % Height of Bumblebee Camera
if cameraPars.canonfl
  cameraPars.CC_HEIGHT35 = 1.5; % Height of Canon Camera
  cameraPars.CC_HEIGHT40 = 1.47; % Height of Canon Camera
end;

% Available resolutions for Canon camera
cameraPars.res_name=cell(5,1); cameraPars.res=zeros(5,2);
cameraPars.res(1,:)=[3648 2736]; cameraPars.res_name{1}='large';
cameraPars.res(2,:)=[2816 2112]; cameraPars.res_name{2}='medium1';
cameraPars.res(3,:)=[2272 1704]; cameraPars.res_name{3}='medium2';
cameraPars.res(4,:)=[1600 1200]; cameraPars.res_name{4}='medium3';
cameraPars.res(5,:)=[640 480];   cameraPars.res_name{5}='small';
% Scaling from bb to given Canon resolution
cameraPars.bbtocan_scaling=3.4*[1.2955 1 0.8068 0.5682 0.2273];

% Use this resolution on Canon camera
if configPars.usingLaptop,
    cameraPars.canon_res=4; % medium3
else
    cameraPars.canon_res=3; % medium2
end

% Zoom scalings for Canon camera
% Scalings from zoom 0 to other zoom levels [0:6]
cameraPars.zoom_scaling=[1 1.09 1.228 1.3065 1.44 1.764 2.08];

% Define centre marker
cameraPars.xx=[-1 1 0 0 0];
cameraPars.yy=[0 0 0 -1 1];
cameraPars.msz=20;

% Using 16bit or 8bit disparity?
cameraPars.is16BitDisp = 1;

% Load predictive map and define memory struct
%load gaze_proto_hires338.mat  % 16000 prototypes from 338 saccades
load gaze_proto_hires600rect.mat  % 16000 prototypes from 600 saccades
gazePars.memory.Ap=Ap;
gazePars.memory.Up=Up;
gazePars.memory.Wa=Wa;
gazePars.memory.Wu=Wu;

% Set parameters for predictive map
gazePars.pars.itype = 4;
gazePars.pars.Nn    = 40;
gazePars.pars.gstd  = 1;

% Connect to robot and get PTU parameters
powerbot('connect','bart.ubcrobots.net',7001);

ptu_lim=powerbot('getlim');
ptu_res=powerbot('getres');

ptu_lim(1:2)=[-90 90];  % Constrain range

cameraPars.ptu_lim = ptu_lim;

state=powerbot('getpos');
pan = state(1);
tilt = state(2);
fprintf('pan=%d tilt=%d\n',pan,tilt);

if configPars.usingLaptop
  camera('connect',1);
  camera('whitebalance',51,30);
  camera('setup_capture',1);
  camera('framerate',15);
  if cameraPars.canonfl
    cameraCapture('start');
    cameraCapture('setsize',cameraPars.res_name{cameraPars.canon_res});
  end;
end;

% Mapping
mappingPars.logFile = fopen('map.log', 'wt');

% Objects
mappingPars.curObjId = 1;
% Each row is [ x y z nPicsTaken freqs(1:36)]
mappingPars.objs = [];
%mappingPars.objs = [2.2047    1.4280   -0.1727    1.0000  1 zeros(1,36);
%                    2.8554   -1.1048    0.1773    1.0000  1 zeros(1,36);
%                    4.0791   -0.7592   -0.3682    1.0000  1 zeros(1,36)];


%objLocs = [ 2.2047    1.4280   -0.1727;
%            2.8554   -1.1048    0.1773;
%            4.0791   -0.7592   -0.3682 ];
%performSlam('add_objects',objLocs);

mappingPars.mapId = 1;

% Planning
planningPars.explored = zeros(240,320); % Binary map of explored frontier points
planningPars.OBSTACLE_THRESHOLD = 0.9;
planningPars.MAX_COST = 16384;
planningPars.exploreDone = false;
planningPars.minViewDist = 14;
planningPars.maxViewDist = 35;

odom = powerbot('getodom')

performSlam('start',odom);
oGrid = performSlam('get_map');

mappingPars.coverageMap = zeros(size(oGrid),'uint8');
%coverageMap = zeros(0,0,'uint8');

% Initially do some random walks and laser scans to build up map
nInitRandWalks = 2;
for i=1:nInitRandWalks
    
  % Random Walk
  [mappingPars, configPars ] = doRandomWalk2(true,mappingPars,planningPars,cameraPars,configPars,gazePars);
  %[mappingPars, configPars ] = doRandomWalk2(true,mappingPars,planningPars,cameraPars,configPars,gazePars);
  
  
  
  positionPars = updatePositionPars();
  
  if configPars.DISPLAY
    % Display map and robot
    figure(10);
    colormap(gray(256));
    image(positionPars.oGrid);
    hold on;
    rGridPos = positionPars.rGridPos;
    plot(rGridPos(2),rGridPos(1),'g+');
    drawnow;
  end;


  % Look at interesting objects
  [configPars mappingPars] = exploreLocation(cameraPars,gazePars,mappingPars,configPars,planningPars);
  
end;

%[mappingPars, configPars ] = doRandomWalk2(1,mappingPars,planningPars,cameraPars,configPars,gazePars);
fprintf('done initial random walks\n');

% Don't random walk for so long
configPars.runTime = 8;
  
% Now enter the main behaviour loop. 
%
% repeat forever...
%
%   (1) Plan next goal
%   (2) Move to it
%   (3) If exploring,
%     (a) grab bumblebee image and pre-process 
%     (b) find K interesting regions
%     (c) for region =  1:K
%       (i) centre region in the Canon camera
%       (ii) zoom the camera so that the region is somewhat tightly cropped
%       (iii) snap an image and save it
%   (4) If looking at an object
%     (a) calculate pan, tilt, zoom to look at object
%     (b) Snap image and save
%
iteration = 0;

probs = [1 .9 .8 .7 .6 .5 .4 .3];

%while(1)
%for i=1:1
while(toc<20*60)
  iteration = iteration + 1;

  oGrid = performSlam('get_map');

%     % Plan a goal
    if iteration <= length(probs)
      planningPars.probExplore = probs(iteration);
    else
      planningPars.probExplore = probs(end);
    end;
    
  % Update Object locations
  mappingPars.objs = updateObjectLocations(mappingPars.objs);
    
  [ goal planningPars.exploreDone ] = globalPlanner(oGrid,mappingPars.objs,planningPars,cameraPars,configPars)
  
  if configPars.DISPLAY
    % Display map and robot
    figure(10);
    colormap(gray(256));
    image(oGrid);
    hold on;
    rGridPos = performSlam('get_curgridpos');
    plot(rGridPos(2),rGridPos(1),'g+');
 
    if goal.type == 'e' || goal.type == 'a'
        frontier = goal.frontier;
        fSize = size(frontier);
        for j=1:fSize(1)
          plot(frontier(j,2),frontier(j,1),'b');
        end;
    end;

    if goal.type == 'e' || goal.type == 'o'
       % Plot path
      plot(goal.path(:,2),goal.path(:,1),'g-');
    end;

    if goal.type == 'o'
        % Looking at an object
        % Plot object location
        objGridPos = performSlam('get_gridpos_from_vecpos',goal.result(7:8));
        plot(objGridPos(2),objGridPos(1),'r.');
    end;
      
    drawnow;
    
    mappingPars.mapId = saveMap(mappingPars.mapId);
  end;
  
  % Always look 40 degrees down while moving
  powerbot('setpos',0,-40);
  
  if goal.type == 'o' || goal.type == 'e'

    success = movePath(oGrid,planningPars,cameraPars,configPars,goal,mappingPars.logFile);
    %success = 1;
    
    % Update Object locations
    mappingPars.objs = updateObjectLocations(mappingPars.objs);

    if success
      if goal.type == 'e'
        % Update map of explored frontiers
        frontier = goal.frontier;
        fSize = size(frontier);
        for j=1:fSize(1)
          planningPars.explored(frontier(j,1),frontier(j,2)) = 1;
        end;
        
        % See if we can look at previous objects
        %[mappingPars.objs configPars.nTotalPics] = lookAtPreviousObjects(mappingPars,cameraPars,configPars,gazePars,ptu_lim);
        % Look at interesting objects
        [configPars mappingPars] = exploreLocation(cameraPars,gazePars,mappingPars,configPars,planningPars)
      elseif goal.type == 'o'
        [mappingPars.objs configPars.nTotalPics] = lookAtPreviousObjects(mappingPars,cameraPars,configPars,gazePars,ptu_lim);
        
        [configPars mappingPars] = exploreLocation(cameraPars,gazePars,mappingPars,configPars,planningPars)
     
      end;
    else % Failed to reach the goal
      % See if we can look at previous objects
      %[mappingPars.objs configPars.nTotalPics] = lookAtPreviousObjects(mappingPars,cameraPars,configPars,gazePars,ptu_lim);
      % Look at interesting objects
      [configPars mappingPars] = exploreLocation(cameraPars,gazePars,mappingPars,configPars,planningPars);

    end;
  
  else

    if goal.type == 'a'
        % We failed to get a path to a frontier, but we picked a direction
        % for random walking
        powerbot('movebotTo',performSlam('get_curvecpos')+[0 0 goal.yaw]);
    end;
    
    % Failed to find a goal, random walk
    [mappingPars, configPars ] =  doRandomWalk2(0,mappingPars,planningPars,cameraPars,configPars,gazePars);
    
    % Update Object locations
    mappingPars.objs = updateObjectLocations(mappingPars.objs);
   
    % See if we can look at previous objects
    %[mappingPars.objs configPars.nTotalPics] = lookAtPreviousObjects(mappingPars,cameraPars,configPars,gazePars,ptu_lim);
    % Look at interesting objects
    [configPars mappingPars] = exploreLocation(cameraPars,gazePars,mappingPars,configPars,planningPars);
  end;
end;

if ~configPars.usingLaptop
%  !chmod -R g+rwx /lci/project/raid1/srvc/DR1/robotImages
%  !chmod -R g+rwx /lci/project/raid1/srvc/DR1/robotImages_bbox
end;

%close log file
fclose(mappingPars.logFile);

%stop SLAM
performSlam('stop');

powerbot('setpos',0,0);
if configPars.usingLaptop
  camera('stop_capture');
  camera('disconnect');
  if cameraPars.canonfl
    cameraCapture('stop');
  end;
end;
powerbot('disconnect');

