%
% This script controls the high level behaviours of the robot during the
% image collection phase of the SRVC
%
% David Meger: May 2, 2007
%
%

% Initialization section...

clear
%clear all;
%close all;
clc;

imageCounter = 1;
K = 4;   % Number of interest points to look at

addpath ../../resources/PEImageGrabbing/
addpath ../stereoCode/
addpath ../matlab_powerbot/
addpath ../gaze/
addpath ../mapping/pmap

SALIENCY_TOOLBOX = '../../resources/SaliencyToolbox';
addpath(genpath(SALIENCY_TOOLBOX));

configFile = '../stereoCode/bb_id5020143.config';

usingLaptop=1; % Are we running on the laptop?

if usingLaptop
  output_path='/home/lciuser/DRI2/robotImages/';
  output_path2='/home/lciuser/DRI2/robotImages_bbox/';
else
  output_path='/lci/project/raid1/srvc/DRI1/robotImages/';
  output_path2='/lci/project/raid1/srvc/DRI1/robotImages_bbox/';
end;

DISPLAY = 1;
bpattern=0;  % For BB-HICOL-40
canonfl=1;   % Use Canon camera if set, otherwise use Bumblebee

% Available resolutions for Canon camera
res_name=cell(4,1);res=zeros(4,2);
res(1,:)=[3648 2736]; res_name{1}='large';
res(2,:)=[2816 2112]; res_name{2}='medium1';
res(3,:)=[2272 1704]; res_name{3}='medium2';
res(4,:)=[640 480];   res_name{4}='small';
bbtocan_scaling=3.4*[1.2955 1 0.8068 0.2273];  % Scaling from bb to
                                               % given Canon resolution

canon_res=3;          % Use this resolution on Canon camera

% Zoom scalings for Canon camera
% Scalings from zoom 0 to other zoom levels [0:6]
zoom_scaling=[1 1.09 1.228 1.3065 1.44 1.764 2.08];

% Load predictive map and define memory struct
%load gaze_proto_hires338.mat  % 16000 prototypes from 338 saccades
load gaze_proto_hires600rect.mat  % 16000 prototypes from 600 saccades
memory.Ap=Ap;
memory.Up=Up;
memory.Wa=Wa;
memory.Wu=Wu;

% Set parameters for predictive map
pars.itype = 4;
pars.Nn    = 40;
pars.gstd  = 1;

% Define centre marker
xx=[-1 1 0 0 0];
yy=[0 0 0 -1 1];
msz=20;

% Using 16bit or 8bit disparity?
is16BitDisp = 0;

% Connect to robot and get PTU parameters
powerbot('connect','bart.ubcrobots.net',7001);

ptu_lim=powerbot('getlim');
ptu_res=powerbot('getres');

ptu_lim(1:2)=[-90 90];  % Constrain range

state=powerbot('getpos');
pan = state(1);
tilt = state(2);
fprintf('pan=%d tilt=%d\n',pan,tilt);

if usingLaptop
  camera('connect',1);
  camera('setup_capture',1);
  camera('framerate',15);
  if canonfl
    cameraCapture('start');
    cameraCapture('setsize',res_name{canon_res});
  end;
end;

% Mapping
logFile = fopen('map.log', 'wt');
crStr = sprintf('\n');
odom = powerbot('getodom');

%log odometry
odomStr = sprintf('o %g %g %g\n',odom);
fwrite(logFile,odomStr,'char');

performSlam('start',odom);

% Now enter the main behaviour loop. 
%
% reapeat forever...
%
%   (1) grab bumblebee image and pre-process 
%   (2) find K interesting regions
%
%   for region =  1:K
%       
%       (3) centre region in the Canon camera
%       (4) zoom the camera so that the region is somewhat tightly cropped
%       (5) snap an image and save it
%   end;
%
%   (6) move the robot randomly to a new position and direction
%
% end;

tic

%for i=1:3
while(toc<15*60)

    % Look down
    tilt0=-35;40; % Look 40deg down
    state0=powerbot('setpos',0,tilt0);
    tilt0=state0(2);

    [img,disp,objectImage] = getImages(tilt0,configFile,usingLaptop,is16BitDisp);         % (1)
    
    % Don't look too far down
    angle_left=tilt0-ptu_lim(3);
    objectImage = pruneObjectImage(objectImage,angle_left);

    if DISPLAY,
        figure(1);clf;
        disp8=disp;disp8(disp8>251)=0;disp8=8*disp8;
        image([ rgb2gray(img) disp8; objectImage zeros(size(objectImage),'uint8')]);
        colormap(gray(256));
        drawnow;
    end;
    
    [ objects ] = getObjects( img, objectImage,K );     % (2)
    
    if DISPLAY
        
        figure(2);clf;
        image(img);
        hold on;
        
        for obj = 1:length(objects)
            
          %plot(objects(obj).centre(2), objects(obj).centre(1),'r+');
          h=plot_objnum(objects);
          set(h,'Color',[1 0 0]);
          bbox = objects(obj).bb;
          h=plot_bbox(bbox,'r-');        
        end;
        
        hold off;
    end;

    %map=performMapping(tilt0,disp,objects,configFile,is16BitDisp,logFile);

    if 0
        figure(10);
        hold on;
        % Plot odometry
        plot(map.odom(1),map.odom(2),'g+');
        % Plot laser
        lSize = size(map.laserPts);
        for i=1:lSize(1)
          plot(map.laserPts(i,1),map.laserPts(i,2),'r');
        end;
        % Plot object centers
        oSize = size(map.objPts);
        for i=1:oSize(1)
          plot(map.objPts(i,1),map.objPts(i,2),'b.'); % plot object center
        end;
        drawnow;
    end;
    
    sc=3.2;     % Scaling from 320x240 to 1024x768
    
    [pl,pr] = getLRPositions(disp,objects,sc);
    
    % Store new states here
    states = cell(length(objects),1);

    % Get current state
    state=powerbot('getpos');

    % Get origin to centre
    if(canonfl)
      wc=[326;512];   % Offset origin (y,x) for Canon camera
    else
      wc=[384;512];   % Image centre (y,x) (for 1024x768 image)
    end
    
    % Find the desired states
    for m=1:length(objects),
      kpl=[pl(:,m);0;0];
      kpr=[pr(:,m);0;0];

      % Call min value searchlim
      %state2 = [0 0];
      state2 = find_saccade(ptu_lim,state,kpl,kpr,memory,pars,wc);
      states{m} = state2;
    end
    
    % Scaling from Bumblebee to current Canon resolution at zoom0 
    sc0=bbtocan_scaling(canon_res);   
    
    wco=res(canon_res,:)'/2;  % Image centre at current res
    
    % Compute the desired zoom:
    %
    % for each object
    % for each possible zoom value
    %     sc_tot=sc*sc0*zoom_scaling(zoom+1);  % Total scaling from 320x240 image
    %     bbox=centreBoundingBox(objects(m),sc_tot,wco);
    %     if upperLeft coord is negative, use previous zoom
    
    zooms = zeros(length(objects),1);
    sc_tots = sc*sc0*zoom_scaling(1)*ones(length(objects),1);
    
    for m=1:length(objects)
     
       for zoom=1:6
        
            sc_tot=sc*sc0*zoom_scaling(zoom+1);
            bbox=centreBoundingBox(objects(m),sc_tot,wco);

            if any(bbox.topLeft < 0 )
                break;
            end
            sc_tots(m)=sc_tot;
            zooms(m) = zoom;
                   
       end;
    end;            
    
    % Use the following 3 lines if you want to keep zoom constant
%     zoom=2;      % Should be estimated instead 
%     sc_tot=sc*sc0*zoom_scaling(zoom+1);  % Total scaling from 320x240 image
%     zooms=ones(size(states))*zoom;
    
    % Now look at objects
    views=getViews(states,configFile,usingLaptop,canonfl,zooms);

    % Save acquired views to disk
    for m=1:length(objects)

      if ~isempty(views{m})
        
        fname = sprintf('%s%d.jpg',output_path,imageCounter);
        if canonfl
          system(sprintf('mv lastframe%d.jpg %s',m,fname));
        else
          imwrite( views{m}, fname, 'JPG' );
        end;
        wco=[size(views{m},1); size(views{m},2)]/2; % Image centre
        bbox=centreBoundingBox(objects(m),sc_tots(m),wco);
        imb=cropImage(views{m},bbox,2);
        fname = sprintf('%s%d.bmp',output_path2,imageCounter);
        imwrite( imb, fname, 'BMP' );
   
        imageCounter = imageCounter + 1;
      end;      
    end;

    if DISPLAY
        figure(3);clf
        for m=1:length(objects),
            subplot(2,2,m)
            if(length(views{m}))
                image(views{m});axis image
                hold on
                h=plot(xx*msz+wco(2),yy*msz+wco(1),'r');
                set(h,'LineWidth',3.0);
                bbox=centreBoundingBox(objects(m),sc_tots(m),wco);
                plot_bbox(bbox,'r-');
            else
                h=text(0,0,'Gaze failed');
                set(h,'FontSize',15);
                set(h,'HorizontalAlignment','center');
                set(h,'VerticalAlignment','middle');
                axis([-1 1 -1 1]);axis off
            end      
            title(['View ' num2str(m)]);        
        end
        drawnow;
    end;
    
    % Move to a new location
    powerbot('movebot');

    iter = 1;
    speeds = [0 0 0];
    while iter <= 10 | abs(speeds(1)) > 0.01 | abs(speeds(3)) > 0.01
      odom = powerbot('getodom');
      laser = powerbot('getlaser');
 
      %log odometry
      odomStr = sprintf('o %g %g %g\n',odom);
      fwrite(logFile,odomStr,'char');

      %log laser
      fwrite(logFile,'l ','char');
      if length(laser) > 0
        laserStr = sprintf('%g ',laser);
        fwrite(logFile,laserStr,'char');
      end;
      fwrite(logFile,crStr,'char');

      if length(odom) == 3 & length(laser) == 361
        performSlam('do_slam', odom, laser);
      end;

      if mod(iter, 10) == 0
        performSlam('get_pmap');
      end;

      speeds = powerbot('getspeeds');
      iter = iter + 1;
    end;

    if DISPLAY
      figure(10);
      imshow(imread('coarse.pgm'));
    end;
end;

toc

if ~usingLaptop
  !chmod -R g+rwx /lci/project/raid1/srvc/DR1/robotImages
  !chmod -R g+rwx /lci/project/raid1/srvc/DR1/robotImages_bbox
end;

%close log file
fclose(logFile);

%stop SLAM
performSlam('get_rmap');
performSlam('stop');
if DISPLAY
  figure(10);
  imshow(imread('fine.pgm'));
end;

powerbot('setpos',0,0);
if usingLaptop
  camera('stop_capture');
  camera('disconnect');
  if canonfl
    cameraCapture('stop');
  end;
end;
powerbot('disconnect');

