%function views=getViews(states,configFile,usingLaptop,canonfl,zooms)
%
% Grab a view for each pan-tilt state in array STATES.
%
% STATES     Cell array of [pan;tilt] states
% CONFIGFILE Config file for Bumblebee camera (has default)
% USINGLAPTOP Control Canon camera directly with laptop if set
% CANONFL    Take snapshots with Canon camera if set (default 0)
% ZOOMS      List of zoom values to use if canonfl==1
%
% Returns:
% VIEWS  Cell array of acquired images, or []
%        if the state is [].
%
%Per-Erik Forssen, May 2007
function views=getViews(states,configFile,usingLaptop,canonfl,zooms)

% Triclops config file
if nargin<2,configFile = '../stereoCode/bb_id5020143.config';end
% Use Canon camera if set
if nargin<3,canonfl=0;end
if nargin<4,zooms=zeros(size(states));end

K=length(states);
bpattern=0;  % For BB-HICOL-40


views=cell(K,1);
for m=1:K,
  if(length(states{m}))
    fprintf('Centering on object %d...\n',m);
    state=powerbot('setpos',states{m}(1),states{m}(2));
    filename = sprintf('lastframe%d.jpg',m);
    if(~canonfl)
      % Use Bumblebee camera
      buf=[];
      while(length(buf)==0)
        if usingLaptop
          buf=camera('getframe');
        else
          buf=powerbot('getframe');
        end;
      end;
      
      % Return rectified left view
      il=bayer2rgbstereo(buf,0,bpattern,2); % De-bayering
      il=colour_rectify(uint8(il*255),configFile,1); % Rectify left field
      views{m}=il;
    else
      % Use Canon camera
      if usingLaptop
        cameraCapture('setzoom',zooms(m));
      else
        powerbot('setzoom',zooms(m));
      end;
      buf=[];
      while(length(buf)==0) 
        if usingLaptop
          buf=cameraCapture('take_photo',filename);
        else
          buf=powerbot('take_photo');
        end;
      end;
      
      if ~usingLaptop
        save_jpeg(buf,filename);
      end;

      icam=imread(filename);
      views{m}=icam;
    end
  end
end

