function [objs nTotalPics] = lookAtPreviousObjects(mappingPars,cameraPars,configPars,gazePars,ptu_lim)
% function for looking at previously seen salient objects again if
%  we are in a good position to do so
%
% objLocs - locations of previously seen objects
% CAMERAHEIGHT - Height of camera used to capture pictures
% ptu_lim - limits of the PTU
% configFile - the Bumblebee configuration file
% usingLaptop - Are we running on the laptop?
% canonfl - Are we using the Canon camera?
%

% Unpack parameters
objs = mappingPars.objs;
MAX_LOOK_DIST = cameraPars.MAX_LOOK_DIST;
canonfl = cameraPars.canonfl;
if canonfl
  CAMERAHEIGHT = cameraPars.CC_HEIGHT35;
else
  CAMERAHEIGHT = cameraPars.BB_HEIGHT35;
end;
configFile = cameraPars.configFile;
numPrevious = configPars.numPrevious;
output_path = configPars.output_path;
output_path2 = configPars.output_path2;
nTotalPics = configPars.nTotalPics;
objectFile1 = configPars.objectFile1;
objectFile2 = configPars.objectFile2;
bpattern = configPars.bpattern;
position_offset = configPars.position_offset;
tilt_offset = configPars.tilt_offset;
usingLaptop = configPars.usingLaptop;
DISPLAY = configPars.DISPLAY;
keypointPath = configPars.keypointPath;
  
% Try to look at the objects
objsSize = size(objs);
%for i=1:1
for i=max(1,objsSize(1)-numPrevious):objsSize(1)
  objLoc = objs(i,1:3);
  
  if all(objLoc == 0)
     continue;
  end;
  objLoc(1) = objLoc(1) - position_offset;

  fprintf('Checking obj%d...\n',i);

  [ state valid ] = computePanTilt( objLoc, cameraPars, configPars );
  if ~valid 
      fprintf('Failed to look back at object.\n');
      continue;
  end;
  tilt = state(2);
  pan = state(1);
  
  %   % Check if the object is in a good position. If we don't know the location,
%   % it is returned as [0 0 0], which is obviously a bad position
%   minDist = (CAMERAHEIGHT-objLoc(3))/tan(abs(ptu_lim(3)-tilt_offset)*pi/180);
%   pan_rad = atan2(objLoc(2),objLoc(1)); % PTU pan
%   horzDist = abs(objLoc(1)/cos(pan_rad));
%   tilt_rad = atan2((CAMERAHEIGHT-objLoc(3)),horzDist);
% 
%   maxDist = MAX_LOOK_DIST*cos(tilt_rad);
%   
%   if objLoc(1) > 0.1 && horzDist > minDist && horzDist < maxDist
%     % Look at it
%     pan = pan_rad *180/pi;
%     tilt = -tilt_rad *180/pi + tilt_offset;
%     if pan < ptu_lim(1) || pan > ptu_lim(2) || tilt < ptu_lim(3) || tilt > ptu_lim(4)
%       fprintf('pan/tilt %d %d exceed limits!\n',pan,tilt);
%       continue;
%     end;

    % increment # of pics we've taken of this object
    objs(i,4) = objs(i,4) + 1;
    nTotalPics = nTotalPics + 1;
    
    filename = sprintf('%s%d.jpg',output_path,nTotalPics);
    fprintf('Taking picture %s...\n',filename);
    powerbot('setpos',state(1),state(2));
    [img,disp,objectImage obstacles] = getImages(state(2),false,configPars,cameraPars);
    % Don't look too far down
    angle_left=tilt-ptu_lim(3);
    objectImage = pruneObjectImage(objectImage,angle_left);

    % Change saliency parameters such that more objects are returned
    saliency_pars=configPars.ss;
    saliency_pars.minthr=1.0;
    saliency_pars.Kmax=15;
    
    [ objects saliencyMap ] = getObjects2( img, objectImage,saliency_pars );
    nObs = length(objects);
    if nObs <= 0
        continue;
    end;
    if 0,
        figure(2);clf;
        image(img);
        hold on;

        for obj = 1:length(objects)

            %plot(objects(obj).centre(2), objects(obj).centre(1),'r+');
            h=plot_objnum(objects);
            set(h,'Color',[1 0 0]);
            bbox = objects(obj).bb;
            h=plot_bbox(bbox,'r-');
        end;

        hold off;

    end
    
    wc = cameraPars.gazeCenter;
    minDist = inf;
    for j=1:nObs
        centre = objects(j).centre;
        dist = norm(centre-wc');
        if dist < minDist
            minDist = dist;
            minInd = j;
        end;
    end;
    object = objects(minInd);
    
    [states zooms sc_tot] = getPanTiltZoom(disp,object,objLoc,cameraPars,gazePars,configPars);
    if(length(states{1}))
      powerbot('setpos',states{1}(1),states{1}(2));
    end;
    if canonfl
      if usingLaptop
        cameraCapture('setzoom',zooms(1));
      else
        powerbot('setzoom',zooms(1));
      end;
    end;

    buf=[];
    if canonfl % Canon camera
      while(isempty(buf))
        if usingLaptop
          buf=cameraCapture('take_photo',filename);
        else
          buf=powerbot('take_photo');
        end;
      end;

      if ~usingLaptop
        save_jpeg(buf,filename);
      end;
      im = imread(filename);
    else % using Bumblebee
      while(isempty(buf))
        if usingLaptop
          buf=camera('getframe');
        else
          buf=powerbot('getframe');
        end;
      end;
      % Save rectified left view
      il=bayer2rgbstereo(buf,0,bpattern,2); % De-bayering
      il=colour_rectify(uint8(il*255),configFile,1); % Rectify left field
      imwrite(il,filename,'JPG','Quality',95);
      im=il;
    end;
    
    wco=[size(im,1); size(im,2)]/2; % Image centre
    bbox=centreBoundingBox(object,sc_tot,wco);
    imb=cropImage(im,bbox,2);
    fname = sprintf('%s%d.jpg',output_path2,nTotalPics);
    imwrite( imb, fname, 'JPG','Quality',95 );

    % Compute SIFT in the background
%     fprintf('Spawning SIFT computation process...\n');
%     fname2 = sprintf('%s%d.sift',output_path2,nTotalPics);
%     system(sprintf('%skeypoints %s %s %d &',keypointPath,fname,fname2,3000)); 

    % Copy to bbox folder (no cropping for now)
    %system(sprintf('cp %s %s',filename,output_path2));
    
    % Write id of object into objects.txt
    entry = sprintf('%d\n',i);
    fwrite(objectFile1,entry,'char');
    fwrite(objectFile2,entry,'char');
    
    pan_rad = state(1)*180/pi;
    % increment bin of angle that we've just seen
    odom = performSlam('get_curvecpos');
    angle = (odom(3) + pan_rad) *18/pi;
    index = 5 + floor(mod(angle,36));
    objs(i,index) = objs(i,index) + 1;

end;

