%function oimg=pruneObjectImage(oimg0,aleft,vfov,crop)
%
% Prune the object image by setting unreachable
% gaze locations to zero.
%
% OIMG0   Input object image
% ALEFT   Angle left to tilt down
% VFOV    Vertical field of view in degrees (default 50)
% CROP    Number of pixels to crop on left and right sides (default 30)
%
%Per-Erik Forssen, May 2007
function oimg=pruneObjectImage(oimg0,aleft,vfov,crop)

if nargin<4,crop=30;end
if nargin<3,vfov=50;end

[rows,cols,ndim]=size(oimg0);
f=rows/2/tan(vfov*pi/180/2); % Vertical focal length
pixels_left=f*tan(aleft*pi/180);
pixel_cutoff=ceil(rows/2+pixels_left);
if(pixel_cutoff>rows) pixel_cutoff=rows;end

oimg=oimg0;    % Copy input image
oimg(pixel_cutoff:end,:)=0;        % Crop lower part
oimg(:,[1:crop end-crop+1:end])=0; % Crop left and right sides
