function [objectImage obstacles] = segmentGround2(img,disp,tilt,isForObstacles,cameraPars,configPars)
%
%
%

% Unpack parameters
position_offset = configPars.position_offset;
tilt_offset = configPars.tilt_offset;
DISPLAY = configPars.DISPLAY;
configFile = cameraPars.configFile;

if abs(tilt+35) < abs(tilt+40)
    CAMERAHEIGHT = cameraPars.BB_HEIGHT35;
else
    CAMERAHEIGHT = cameraPars.BB_HEIGHT40;
end;

is16BitDisp = cameraPars.is16BitDisp;

% List of [row col] of every pixel in the image
[imRows imCols] = size(disp);
rowCols = [];
for i=1:imRows-1
    rowCols = [rowCols; ones(imCols,1)*i (1:imCols)'];
end

% Get pixels in camera coordinates
if is16BitDisp
  pc = triclops_getXYZ16(rowCols,disp, configFile);
else
  pc = triclops_getXYZ(rowCols,disp, configFile);
end;

tilt = tilt-tilt_offset;
tiltRad = tilt * pi/180;

% Matrix to rotate 3D coordinates from camera to robot frame
Rx_cam = [1    0              0; 
          0    cos(tiltRad)   -sin(tiltRad);
          0    sin(tiltRad)   cos(tiltRad)];

% Find points that are above the ground plane
nPix = imRows*imCols;
obstacles = zeros(nPix,5); % Preallocate for efficiency
pcSize = size(pc);
nObs = 0;
%figure(9);segmentGround2
%hold on;
for i=1:pcSize(1)
    if isForObstacles
        THRESHOLD = 0.5;
    else
        THRESHOLD = 0.08-0.03*(rowCols(i,1)/240);%0.1;%
    end;
    pt = pc(i,:);
    val = pt(3);
    if val ~= -1
      %Translate object center to robot coordinates
      yzx_r = Rx_cam*pt';
      xyz_r = [yzx_r(3)+position_offset -yzx_r(1) CAMERAHEIGHT-yzx_r(2)];
      %plot3(xyz_r(1),xyz_r(2),xyz_r(3),'r.');
      if xyz_r(3) > THRESHOLD 
        nObs = nObs+1;
        obstacles(nObs,:) = [rowCols(i,:) xyz_r];
      end;
    end;
end;
% Truncate to actual # of obstacles
obstacles = obstacles(1:nObs,:);
size(obstacles)

if DISPLAY
    figure(6);
    hold on;
    plot(obstacles(:,1),obstacles(:,5),'r.');
    grid on;
    drawnow;
%   % plot obstacles
%   figure(1);
%   image(img);
%   im2 = img;
%   im2(sub2ind(size(im2),[obstacles(:,1),ones(nObs(1),1)],[obstacles(:,2),ones(nObs(1),1)]))=255;
%   %plot(obstacles(:,2),obstacles(:,1),'r.');
%   image(im2);
%   drawnow;
end



objectImage = zeros(imRows,imCols,'uint8');
objectImage(sub2ind(size(objectImage),obstacles(:,1),obstacles(:,2)))=255;
objectImage = im2bw(objectImage);
rBlur = 5;
rErode = 4;

objectImage2 = imdilate(objectImage,strel('disk',rBlur));
objectImage2=imerode(objectImage2,strel('disk',rBlur+rErode));
objectImage2 = imdilate(objectImage2,strel('disk',rBlur+rErode));
objectImage = min(objectImage,objectImage2);  % Do not add pixels

% Remove small stuff in obstacles array
il=sub2ind(size(objectImage),obstacles(:,1),obstacles(:,2));
keepl=find(objectImage(il));
obstacles=obstacles(keepl,:);

if DISPLAY
  figure(4);
  im2 = img;
  indices = find(objectImage ~=0);
  [rows cols] = ind2sub(size(objectImage),indices);
  nObs = size(rows);
  
  im2(indices+0*prod(size(im2(:,:,1)))) = 255;
  im2(indices+1*prod(size(im2(:,:,1)))) = 255;
  im2(indices+2*prod(size(im2(:,:,1)))) = 255;
  %im2(rows,cols,1:3)=255;
  %im2(sub2ind(size(im2),[rows,ones(nObs(1),1)],[cols,ones(nObs(1),1)]))=255;
  %plot(obstacles(:,2),obstacles(:,1),'r.');
  image(im2);
  drawnow;
end