function map = plotMap( logFile )
% function map = plotMap( logFile )
% Plot map using readings from logFile
%
% log file format:
% All coordinates are in the robot's frame 
% o x y yaw - Odometry: position coordinates (x,y) and yaw
% l x1 y1 x2 y2 ... - Laser: xi yi are coordinates of
%                         point i detected by laser
%
fid = fopen(logFile);
hold on;

% Plot salient objects onto map
tilt = -35;
tilt = tilt-5; % Add offset of -5, determined from PTU calibration
tiltRad = tilt * pi/180;

% Matrix to rotate 3D coordinates from camera to robot frame
Rx_cam = [1    0              0; 
          0    cos(tiltRad)   -sin(tiltRad);
          0    sin(tiltRad)   cos(tiltRad)];

while feof(fid) == 0
  line = fgetl(fid);
  if line(1) == 'o' % Odometry
    data = sscanf(line(3:end), '%f')';
    newestOdom = data; % Remember the newest odometry reading

    yaw = newestOdom(3); % yaw in radians
    %Matrix to rotate 2d coordinates from robot to global frame
    R_r = [cos(yaw), -sin(yaw); sin(yaw), cos(yaw)];

    plot(newestOdom(1),newestOdom(2),'g+');
  elseif line(1) == 'l' % Laser
    data = sscanf(line(3:end), '%f %f ')';
    
    count = length(data);
    for i=1:2:count
      xy_r = [data(i), data(i+1)]';
      xy = R_r*xy_r;
      xy = xy + newestOdom(1:2)';
      
      plot(xy(1),xy(2),'r');
    end; % for
  else % Objects
    data = sscanf(line(3:end), '%f %f %f ');

    % Translate object center to global coordinates
    yzx_r = Rx_cam*data;
    xy = R_r*[yzx_r(3) -yzx_r(1)]';
    xy = xy + newestOdom(1:2)';
    plot(xy(1),xy(2),'b.');
  end; % if
end; % while
fclose(fid);
