/*
 *  Simple access to bumblebee camera via video1394 device
 */

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <libraw1394/raw1394.h>
#include <libdc1394/dc1394_control.h>
#include "pnmio.h"
#include "bb_mini.h"
#include "string1394.h"
                                                            
u_int64_t g_guid = 0; /* guid for camera, or 0 for first camera on bus */
                                                                                
int BB_save_image(char *filename, dc1394_cameracapture *camera) {
  FILE *fid;
  fid=pnm_writehead(filename,RPPM_FORMAT,camera->frame_height,
		    camera->frame_width);
  fwrite(camera->capture_buffer,sizeof(char),
	 camera->frame_height*camera->frame_width*3,fid);
  fclose(fid);  
  printf("Saved %dbpp %dx%d image as '%s'\n",24,camera->frame_width,
	   camera->frame_height,filename);
  return 0;
}

int BB_find_camera(raw1394handle_t *rhandle,dc1394_cameracapture *camera,int infofl) {
  raw1394handle_t handle;
  nodeid_t * camera_nodes = NULL;
  dc1394_camerainfo info;
  struct raw1394_portinfo ports[MAX_PORTS];
  int numPorts = 0;
  int numCameras = 0;
  int found = 0;
  int i,j;

 /* Open ohci and asign handle to it */
  handle = raw1394_new_handle();
  if (handle==NULL)
  {
    fprintf(stderr,"Unable to aquire a raw1394 handle\n\n");
    fprintf(stderr,"Please check \n");
    fprintf(stderr,"  - if the kernel modules `ieee1394',`raw1394' and `ohci1394' are loaded \n");
    fprintf(stderr,"  - if you have read/write access to /dev/raw1394\n\n");
    return -1;
  }
  /* get the number of ports (cards) */
  numPorts = raw1394_get_port_info(handle, ports, numPorts);
  raw1394_destroy_handle(handle);

  handle = NULL;

  for (j = 0; j < MAX_RESETS && found == 0; j++)
  {
    /* look across all ports for cameras */
    for (i = 0; i < numPorts && found == 0; i++)
    {
      if (handle != NULL)
        dc1394_destroy_handle(handle);
      handle = dc1394_create_handle(i);
      if (handle == NULL)
      {
        fprintf( stderr, "Unable to aquire a raw1394 handle for port %i\n", i);
        return -1;
      }
      numCameras = 0;
      camera_nodes = dc1394_get_camera_nodes(handle, &numCameras, 0);
      if (numCameras > 0)
      {
        if (g_guid == 0)
        {
          /* use the first camera found */
          camera->node = camera_nodes[0];
          if (dc1394_get_camera_info(handle, camera_nodes[0], &info) == DC1394_SUCCESS)
	    if(infofl) dc1394_print_camera_info(&info);
          found = 1;
        }
        else
        {
          /* attempt to locate camera by guid */
          int k;
          for (k = 0; k < numCameras && found == 0; k++)
          {
            if (dc1394_get_camera_info(handle, camera_nodes[k], &info) == DC1394_SUCCESS)
            {
              if (info.euid_64 == g_guid)
              {
                if(infofl) dc1394_print_camera_info(&info);
                camera->node = camera_nodes[k];
                found = 1;
              }
            }
          }
        }
        if (found == 1)
        {
          /* camera can not be root--highest order node */
          if (camera->node == raw1394_get_nodecount(handle)-1)
          {
            /* reset and retry if root */
            raw1394_reset_bus(handle);
            sleep(2);
            found = 0;
          }
        }
        dc1394_free_camera_nodes(camera_nodes);
      } /* cameras >0 */
    } /* next port */
  } /* next reset retry */

  if (found == 0 && g_guid != 0)
  {
    fprintf(stderr,"Unable to locate camera node by guid\n");
    return -1;
  }
  else if (numCameras == 0)
  {
    fprintf(stderr,"no cameras found :(\n");
    dc1394_destroy_handle(handle);
    return -1;
  }
  if (j == MAX_RESETS)
  {
    fprintf(stderr, "failed to not make camera root node :(\n");
    dc1394_destroy_handle(handle);
    return -1;
  }
  rhandle[0]=handle;  /* Return handle */
}

int BB_release(raw1394handle_t handle,dc1394_cameracapture *camera) {
  /* Close camera */
  dc1394_dma_unlisten(handle,camera);
  dc1394_dma_release_camera(handle,camera);
  dc1394_destroy_handle(handle);
}

/*
 * Now we've got a camera, setup capture
 * Includes hardcoded settings of framerate and
 * video mode for BBCOL-40 and BBHICOL-40
 */
int BB_setup_capture(raw1394handle_t handle,dc1394_cameracapture *camera,char * dma_dev_name) {

  dc1394_miscinfo misc_info;

 /* Read current settings */
  if(dc1394_get_camera_misc_info(handle,camera->node,&misc_info)!=DC1394_SUCCESS) {
    fprintf(stderr,"Could not get camera misc info\n");
    dc1394_destroy_handle(handle);
    return -1;
  }

  if(misc_info.format==FORMAT_VGA_NONCOMPRESSED) {
    /* Setting for old bumblebee BB_COL_40 */
    misc_info.mode=MODE_640x480_MONO16;
    misc_info.framerate=FRAMERATE_30;
  }

  if(misc_info.format==FORMAT_SVGA_NONCOMPRESSED_1) {
    /* Setting for new bumblebee BB_HICOL_40 */
    misc_info.mode= MODE_1024x768_MONO16;
    misc_info.framerate=FRAMERATE_15;
  }

  printf("minfo.format %s\n",getstr_format(misc_info.format));
  printf("     .mode %s\n",getstr_mode(misc_info.mode));
  printf("     .framerate %s\n",getstr_framerate(misc_info.framerate));
  if(misc_info.is_iso_on) {
    printf("     .is_iso_on true\n");
    printf("     .iso_channel %d\n",misc_info.iso_channel);
    printf("     .iso_speed %s\n",getstr_speed(misc_info.iso_speed));
  } else {
    printf("     .is_iso_on false\n");
  }

  if (dc1394_dma_setup_capture(handle, camera->node,
			       misc_info.iso_channel, /* channel */ 
			       misc_info.format,      /* format */
			       misc_info.mode,        /* video mode */
			       misc_info.iso_speed,   /* speed */
			       misc_info.framerate,   /* framerate */
			       DC1394_BUFFERS,
			       DROP_FRAMES, 
			       dma_dev_name,
			       camera)!=DC1394_SUCCESS) 
  {
    fprintf(stderr,"unable to setup camera-\n"
	    "check line %d of %s to make sure\n"
	    "that the video mode,framerate and format are\n"
	    "supported by your camera\n",
	    __LINE__,__FILE__);
    BB_release(handle,camera);
    return -1;
  }
  return 0;
}

int BB_setwhitebalance(raw1394handle_t handle,dc1394_cameracapture *camera,
		       int whitebal_ub,int whitebal_vr) {
  if(dc1394_set_white_balance(handle,camera->node,whitebal_ub,whitebal_vr)) {
    return 1;
  } else {
    fprintf(stderr,"Failed to set whitebalance\n");
    return 0;
  }
}

int BB_start_iso_transmission(raw1394handle_t handle,dc1394_cameracapture *camera) {
  /* have the camera start sending us data */
  if (dc1394_start_iso_transmission(handle,camera->node)
      !=DC1394_SUCCESS) 
  {
    fprintf(stderr,"unable to start camera iso transmission\n");
    BB_release(handle,camera);
    return -1;
  }
  return 0;
}

int BB_single_capture(raw1394handle_t handle,dc1394_cameracapture *camera) {
  /* capture one frame */
  if (dc1394_dma_single_capture(camera)!=DC1394_SUCCESS) 
  {
    fprintf(stderr,"unable to capture a frame\n");
    BB_release(handle,camera);
    return -1;
  }
  return 0;
}

int BB_buffer_free(dc1394_cameracapture *camera) {
   return dc1394_dma_done_with_buffer(camera); 
}
