function [cells knownFrontiers] = addCellsToFrontier(i,j,eGrid,gridSize,cells,unexpVal,knownFrontiers)
% Function to add a cell to the frontier if it is adjacent to an 
% unexplored cell
%
% eGrid(i j) - grid cell to add
% gridSize - size(eGrid)
% cells - cells currently in the frontier
% unexpVal - value for an unexplored cell
% knownFrontiers - binary map of known frontier points
%

% Check if cell is unoccupied and is not already in the frontier
fSize = size(cells);
if eGrid(i,j) == 254 && knownFrontiers(i,j) == 0 && fSize(1) < 50

  % Check if any adjacent cells are unexplored
  if i == 1 && j == 1
    isFrontierCell=eGrid(i,j+1) == unexpVal || eGrid(i+1,j) == unexpVal || eGrid(i+1,j+1) == unexpVal;
  elseif i == gridSize(1) && j == gridSize(2)
    isFrontierCell=eGrid(i-1,j-1) == unexpVal || eGrid(i-1,j) == unexpVal || eGrid(i,j-1) == unexpVal;
  elseif i == 1 && j == gridSize(2)
    isFrontierCell=eGrid(i,j-1) == unexpVal || eGrid(i+1,j-1) == unexpVal || eGrid(i+1,j) == unexpVal;
  elseif i == gridSize(1) && j == 1
    isFrontierCell=eGrid(i-1,j) == unexpVal || eGrid(i-1,j+1) == unexpVal || eGrid(i,j+1) == unexpVal;
  elseif i == 1
    isFrontierCell=eGrid(i,j-1) == unexpVal || eGrid(i,j+1) == unexpVal || ...
                   eGrid(i+1,j-1) == unexpVal || eGrid(i+1,j) == unexpVal || eGrid(i+1,j+1) == unexpVal;
  elseif i == gridSize(1)
    isFrontierCell=eGrid(i-1,j-1) == unexpVal || eGrid(i-1,j) == unexpVal || eGrid(i-1,j+1) == unexpVal || ...
                   eGrid(i,j-1) == unexpVal || eGrid(i,j+1) == unexpVal;
  elseif j == 1
    isFrontierCell=eGrid(i-1,j) == unexpVal || eGrid(i-1,j+1) == unexpVal || ...
                   eGrid(i,j+1) == unexpVal || ...
                   eGrid(i+1,j) == unexpVal || eGrid(i+1,j+1) == unexpVal;
  elseif j == gridSize(2)
    isFrontierCell=eGrid(i-1,j-1) == unexpVal || eGrid(i-1,j) == unexpVal || ...
                   eGrid(i,j-1) == unexpVal || ...
                   eGrid(i+1,j-1) == unexpVal || eGrid(i+1,j) == unexpVal;
  else
    isFrontierCell=eGrid(i-1,j-1) == unexpVal || eGrid(i-1,j) == unexpVal || eGrid(i-1,j+1) == unexpVal || ...
                   eGrid(i,j-1) == unexpVal || eGrid(i,j+1) == unexpVal || ...
                   eGrid(i+1,j-1) == unexpVal || eGrid(i+1,j) == unexpVal || eGrid(i+1,j+1) == unexpVal;
  end;
  
  if isFrontierCell
    % This is a frontier cell, add it
    cells = [cells; i j];
    
    % Add this cell to known frontiers
    knownFrontiers(i,j) = 1;

    % Recursive calls to adjacent cells
    if i < gridSize(1)
      [cells knownFrontiers]=addCellsToFrontier(i+1,j,eGrid,gridSize,cells,unexpVal,knownFrontiers);
    end;
    if i > 1
      [cells knownFrontiers]=addCellsToFrontier(i-1,j,eGrid,gridSize,cells,unexpVal,knownFrontiers);
    end;
    if j < gridSize(2)
      [cells knownFrontiers]=addCellsToFrontier(i, j+1,eGrid,gridSize,cells,unexpVal,knownFrontiers);
    end;
    if j > 1
      [cells knownFrontiers]=addCellsToFrontier(i,j-1,eGrid,gridSize,cells,unexpVal,knownFrontiers);
    end;
    if i < gridSize(1) && j < gridSize(2)
      [cells knownFrontiers]=addCellsToFrontier(i+1,j+1,eGrid,gridSize,cells,unexpVal,knownFrontiers);
    end;
    if i > 1 && j < gridSize(2)
      [cells knownFrontiers]=addCellsToFrontier(i-1,j+1,eGrid,gridSize,cells,unexpVal,knownFrontiers);
    end;
    if i < gridSize(1) && j > 1
      [cells knownFrontiers]=addCellsToFrontier(i+1,j-1,eGrid,gridSize,cells,unexpVal,knownFrontiers);
    end;
    if i > 1 && j > 1
      [cells knownFrontiers]=addCellsToFrontier(i-1,j-1,eGrid,gridSize,cells,unexpVal,knownFrontiers);
    end;
  end;
end;
