function [dists parents nFreeIndices free_indices dist_map] = buildPathMap(oGrid,rGridPos,OBSTACLE_THRESHOLD)
% Build a map of costs from the current robot location to every other
% location on the map
%
% oGrid - the occupancy grid map
% rGridPos - The robot's current position on the occupancy grid
% OBSTACLE_THRESHOLD - threshold above which grid cells are confidently
%                     ground
%

% Threshold map so only true obstacles look like obstacles
obstacle_map = im2bw(oGrid,OBSTACLE_THRESHOLD);

% figure(11);
% imagesc(obstacle_map);
% title('Obstacles');

obstacle_map = imdilate(1-obstacle_map,strel('disk',5));

% Robot might be closer than 0.6m to objects, area 0.6 around robot
% should also be free
if obstacle_map(rGridPos(1),rGridPos(2)) == 1
  fprintf('Robot is in an obstacle!!\n');

  disk5 = [ 1 1 0 0 0 0 0 1 1 ;
            1 0 0 0 0 0 0 0 1 ;
            0 0 0 0 0 0 0 0 0 ;
            0 0 0 0 0 0 0 0 0 ;
            0 0 0 0 0 0 0 0 0 ;
            0 0 0 0 0 0 0 0 0 ;
            0 0 0 0 0 0 0 0 0 ;
            1 0 0 0 0 0 0 0 1 ;
            1 1 0 0 0 0 0 1 1 ];
%   disk6 = [ 1 1 0 0 0 0 0 0 0 1 1 ;
%             1 0 0 0 0 0 0 0 0 0 1 ;
%             0 0 0 0 0 0 0 0 0 0 0 ;
%             0 0 0 0 0 0 0 0 0 0 0 ;
%             0 0 0 0 0 0 0 0 0 0 0 ;
%             0 0 0 0 0 0 0 0 0 0 0 ;
%             0 0 0 0 0 0 0 0 0 0 0 ;
%             0 0 0 0 0 0 0 0 0 0 0 ;
%             0 0 0 0 0 0 0 0 0 0 0 ;
%             1 0 0 0 0 0 0 0 0 0 1 ;
%             1 1 0 0 0 0 0 0 0 1 1 ];
  rowRange = rGridPos(1)-4:rGridPos(1)+4;
  colRange = rGridPos(2)-4:rGridPos(2)+4;
  obstacle_map(rowRange,colRange)=obstacle_map(rowRange,colRange).*disk5;
end;

free_indices = find(obstacle_map~=1);
nFreeIndices = length(free_indices);

% Distance transform, calculate distance to obstacle for every point
dist_map = bwdist(obstacle_map);

scaled_im = 100./(dist_map+ 1) + 1;

% figure(12);
% imagesc(scaled_im);
% hold on;
% title('Planning Cost Function');

scaled_im(obstacle_map==1) = intmax('int32');

% Calculate paths and distances to all locations
[ dists parents ] = gridAStar( int32(scaled_im), int32(rGridPos), int32(rGridPos) );