function [newPath pathSize] = checkForObstacles(wp,orgPath,oGrid,planningPars)
% Check if there are obstacles along the path. If so, try to re-plan a
% path to the goal
%
% wp - The waypoint we are currently on
% orgPath - Original series of positions, expressed in 
%           occupancy grid coordinates
% oGrid - the occupancy grid map
%

% Unpack parameters
OBSTACLE_THRESHOLD = planningPars.OBSTACLE_THRESHOLD;
MAX_COST = planningPars.MAX_COST;

% Check if our path will make us collide into obstacles
path = orgPath(wp:end,:);
pathMap = im2bw(zeros(size(oGrid)));
pathMap(sub2ind(size(oGrid),path(:,1),path(:,2))) = 1;
pathMap = imdilate(pathMap,strel('disk',5));
nObstacleCells = nnz(oGrid(pathMap) < OBSTACLE_THRESHOLD*255);
if nObstacleCells > 0
    fprintf('Visual obstacles detected along path!\n');
    % Stop robot
    powerbot('turnbot',0);
    
    % Re-plan
    rGridPos = performSlam('get_curgridpos');

    [dists parents nFreeIndices] = buildPathMap(oGrid,rGridPos,OBSTACLE_THRESHOLD);

    pathSize = size(path);
    for i=pathSize(1):-1:max(pathSize(1)-2,1)
        if dists(path(i,1),path(i,2)) < MAX_COST
            newPath = [orgPath(1:wp-1,:);
                       buildGridPath(parents,rGridPos,path(i,:))];
            pathSize = size(newPath);
            fprintf('Planned new path around visual obstacles\n');
            return;
        end;
    end;
    fprintf('Area near goal is occupied!\n');
    newPath = [];
    pathSize = size(newPath);

else % No need to modify path
    newPath = orgPath;
    pathSize = size(newPath);
end;

