function [frontiers lengths] = getFrontiers(oGrid,unexpVal,minLength,knownFrontiers)
% Uses the occupancy grid to find frontiers to unexplored regions
% (See "A Frontier-Based Approach for Autonomous Exploration" by Brian Yamauchi)
%
% oGrid - The Occupancy Grid
% unexpVal- value of unexplored cell
% minLength - Minimum length of frontier to be considered (in pixels)
% knownFrontiers - binary map of explored frontier points
%

% Build evidence grid
gridSize = size(oGrid);
eGrid = zeros(gridSize);
eGrid(find(oGrid>=127))=unexpVal;
eGrid(find(oGrid>unexpVal))=254;
% for i=1:gridSize(1)
%   for j=1:gridSize(2)
%     if oGrid(i,j) > unexpVal
%       eGrid(i,j) = 254;
%     elseif oGrid(i,j) >= 127
%       eGrid(i,j) = unexpVal;
%     end;
%   end;
% end;

if 1
    % Remove unexplored grid locations neighbouring a wall
    wallMask=double(eGrid==0);
    wallMask=imdilate(wallMask,strel('square',3));
    eGrid(find(wallMask.*(eGrid==unexpVal)))=0;
end;

% Find frontiers
frontiers = [];
lengths = [];
count = 1;
for i=1:gridSize(1)
  for j=1:gridSize(2)
    if eGrid(i,j) == 254 && knownFrontiers(i,j) == 0
      frontier = addCellsToFrontier(i,j,eGrid,gridSize,[],unexpVal,knownFrontiers);

      % Only accept frontiers >= minLength
      fSize = size(frontier);
      if fSize(1) > 1
        fLength = norm(max(frontier)-min(frontier));
        if fLength >= minLength
          frontiers{count} = frontier;
          % Add frontiers to list of known frontiers
          for k=1:fSize(1)
            knownFrontiers(frontier(k,1),frontier(k,2)) = 1;
          end;
          lengths = [lengths; fLength];
          count = count + 1;
        end;
      end;
    end;
  end;
end;

% Find representative points for each frontier
for i=1:count-1
  frontier = frontiers{i};

  % Find the 'center of gravity'
  center = round(mean(frontier));

  % calculate distances from each point to the center
  minDist = inf;
  fSize = size(frontier);
  for j=1:fSize(1)
    dist = norm(frontier(j,:)-center);
    if dist < minDist
      minDist = dist;
      closest = frontier(j,:);
    end;
  end;

  % Put the center and the closest frontier point in front
  frontiers{i} = [center; closest; frontier];
end;

