function [ binScores wedgeMasks ] = getUnseenViews( mappingPars, planningPars )

% Find views around our current location that have not yet been covered by
% the camera

positionPars = updatePositionPars();

oGrid = positionPars.oGrid;
gridPos = positionPars.rGridPos;
rPose = positionPars.rPose;
OBSTACLE_THRESHOLD = planningPars.OBSTACLE_THRESHOLD;
minDist = planningPars.minViewDist;
maxDist = planningPars.maxViewDist;
coverageMap = mappingPars.coverageMap;

%oGrid = imread('../planning/map.pgm');
%gridPos = [100 150];

%oGrid(90:100,190:200)=0;

figure(45);
colormap(gray(256));imagesc(oGrid);
hold on;
plot(gridPos(2),gridPos(1),'g+');

[rows,cols] = size(oGrid);
obstacle_map = im2bw(oGrid,0.5);

% First compute the bounding area of the map (map_region)

figure(47);
subplot(2,2,1);
colormap(gray(256));
imagesc(obstacle_map);
subplot(2,2,2);
omapdilate=imdilate(obstacle_map,strel('disk',6));
colormap(gray(256));
imagesc(omapdilate);
subplot(2,2,3);
map_region=imerode(omapdilate,strel('disk',6));
colormap(gray(256));
imagesc(map_region);
subplot(2,2,4);
internal_obstacles = map_region-obstacle_map;
colormap(gray(256));
imagesc(internal_obstacles);

% Now find area inside map region that hasn't been seen so far

unseen_regions = uint8(map_region) .* ( 255 - coverageMap );

imagesc(unseen_regions);

largeMask = zeros([rows cols],'uint8');
wedgeMask = zeros([rows cols],'uint8');
smallMask = ones([rows cols],'uint8');

cl=-minDist:minDist;
[x y] = meshgrid(cl,cl);
[ir ic]=find(x.^2+y.^2<minDist^2);
ir=cl(ir)+gridPos(1);
ic=cl(ic)+gridPos(2);
validl=find((ir>0)&(ir<rows)&(ic>0)&(ic<cols));
il=sub2ind([rows cols],ir(validl),ic(validl));
smallMask(il)=0;

cl=-maxDist:maxDist;
[x y] = meshgrid(cl,cl);
[ir ic]=find(x.^2+y.^2<maxDist^2);
ir=cl(ir)+gridPos(1);
ic=cl(ic)+gridPos(2);
validl=find((ir>0)&(ir<rows)&(ic>0)&(ic<cols));
il=sub2ind([rows cols],ir(validl),ic(validl));
largeMask(il)=1;

visible_regions = unseen_regions .* largeMask;
visible_regions = visible_regions .* smallMask;

figure(48);
colormap(gray(256));
imagesc(unseen_regions);

yaw = rPose(3);

figure(46);
colormap(gray(256));

% Return the wedge masks
wedgeMasks = zeros(rows,cols,6);

for angleBin = -3:2
    
    wedgeMask(:) = 0;
    minRange = angleBin * pi/3;
    maxRange = minRange + pi/3;
    
    % This line transformed things into the local robot frame. Instead,
    % I'll leave them global
    %[ir ic] = find( x.^2+y.^2<maxDist^2 & thetapp(atan2(y,x)-yaw)>minRange & thetapp(atan2(y,x)-yaw)<maxRange);
    [ir ic] = find( x.^2+y.^2<maxDist^2 & atan2(y,x)>minRange & atan2(y,x)<maxRange);
    ir=cl(ir)+gridPos(1);
    ic=cl(ic)+gridPos(2);
    validl=find((ir>0)&(ir<rows)&(ic>0)&(ic<cols));
    il=sub2ind([rows cols],ir(validl),ic(validl));
    wedgeMask(il)=1;
    
    binScores(angleBin+4) = sum(sum(visible_regions.*wedgeMask));
    subplot(3,2,angleBin+4);
    imagesc(visible_regions.*wedgeMask);    
    
    % Return the wedge masks
    wedgeMasks(:,:,angleBin+4) = wedgeMask;
end;


%[x y] = find(obstacle_map~=0);
%[k,a] = convhull(x, y);
%plot(y(k),x(k),'r-',y,x,'b+')

% neighborhood = zeros(size(oGrid),'uint8');
% [x,y] = meshgrid( max(1,gridPos(1)-radius):min(rows,gridPos(1)+radius),max(1,gridPos(2)-radius):min(cols,gridPos(2)+radius));
% idx = [ x(:) y(:) ];
% dists = idx - repmat(gridPos,length(idx),1);
% dists = dists(:,1).^2 + dists(:,2).^2;
% idx = find( dists < radius^2 );
% %neighborhood(ind2sub([rows cols],idx)) = 255;
% neighborhood(idx)=255;
% imagesc(neighborhood);


