function [path,distance] = gridAStar( map, start, goal )

[height width] = size(map);

grid = gridBuildAdjacencyLists( height,width);

% figure(2);
% hold on;

% examined = zeros(height, width,'uint8');
% examined(start(1), start(2)) = 255;

bestCosts = bitmax * ones(height,width);
bestPaths = zeros(height,width,2);

% Initialize the open and closed lists and all backpointers
	
bestCosts(start(1),start(2)) = 0;
bestPaths(start(1),start(2),:) = [ 0 0 ];

% Calculate start's heuristic score:
f_est = norm( start-goal,2 );

open = [ f_est 0 start ];
closed = [];

while 1
   
   % Next node on open list to process is always at the top, because we
   % keep this list sorted. Remove this node from the list and process all
   % of it's neighbors.
   
   current_path_cost = open(1,2);
   current_row = open(1,3);
   current_col = open(1,4);
   
   open = open( 2:end, : );
   closed = [ closed; current_row current_col ];
   
   %neighbors = gridFindNewReachableNeighbors( current_row, current_col, height, width, closed,open );
   neighbors = grid.adjLists{current_row,current_col}(find(grid.valid{current_row,current_col}),:);
   
   [ numNeighbors, junk ] = size( neighbors );
   
   for idxNeighbors = 1:numNeighbors
              
       neighbor = neighbors(idxNeighbors,:);
       
       % Remove the neighbor pointers which would cause us to search back
       % to the previous node, and also that link current neighbors
       
       grid.valid{neighbor(1),neighbor(2)}(mod((neighbor(3)-1)+4,8)+1)=0;
       grid.valid{neighbor(1),neighbor(2)}(mod((neighbor(3)-1)+1,8)+1)=0;
       grid.valid{neighbor(1),neighbor(2)}(mod((neighbor(3)-1)-1,8)+1)=0;
       
       
%        open;
%        examined(neighbor(1),neighbor(2)) = 255;
%        
%        image(examined);
%        drawnow;
       
       if map(neighbor(1),neighbor(2)) == 255
           g = current_path_cost + norm([neighbor(1) neighbor(2)]-[current_row current_col],2);
       else
           g = bitmax;
       end;
              
       % If this path to the neighbor is better than the previous, update
       % the cost and path, and put the node back on the open list.
       % Possibly need to remove it's previous entry in the open list!
       if( g < bestCosts(neighbor(1),neighbor(2) ) )

           bestCosts(neighbor(1),neighbor(2)) = g;
           bestPaths(neighbor(1),neighbor(2),:) = [ current_row current_col ];
           
           % Check if we have reached the goal. With A* and an admissible
           % heuristic, we can stop as soon as this occurs!
           
           if [ neighbor(1) neighbor(2)] == goal
               
               distance = g;
               
               % Compute the path backwards
               path = [];
               current = neighbor;
               
               while current(1) ~= start(1) || current(2) ~= start(2)
                   
                   path = [ bestPaths(current(1),current(2),:); path ];
                   current = bestPaths(current(1),current(2),:);
               end;
               
               return;
           end;              
           
           % Check if already on open, if so, remove
%            previousIndex = find( open(:,3) == neighbor(1) & open(:,4) == neighbor(2) );
%            
%            if ~isempty( previousIndex )
%                
%                open = [ open(1:previousIndex-1,:); open(previousIndex+1:end,:) ];
%            end;
         
           % We update and put on open list (in sorted order based on f_est
           % please!):
           h = norm( [neighbor(1) neighbor(2)] - goal, 2 );
           f_est = g + h;
           
           position = 1;
           [ numOpen junk ] = size( open );
           
%            open = [ [f_est g neighbor]; open ];
           
           if isempty(open)
               
               open = [ f_est g neighbor ];
           else
           
               while( f_est > open(position,1) )
                   position = position + 1;

                   if position > numOpen
                       break;
                   end;
               end;
               
               % At this point, position holds the correct place to insert our
               % new cell in the priority queue
               open = [ open(1:position-1,:); [ f_est g neighbor ]; open(position:end,:) ];
           end;
       end;
   end;
end;
           
           
           
       
       
       
       