%
% This simple script drives the robot based on laser information and tests
% planning algorithms. I would like this script to explicitely avoid using
% non-simulatable functions (ie, pieces of robot hardware which aren't
% supported in Stage), so that we can test things out very quickly without
% bothering the real hardware. Actual testing on the robot can be done
% using SRVC_phase2.m
%
% David Meger - June 14th 2007
%

% Initialization section...

clear all;
close all;
clc;

addpath ../mlpb_pb/
addpath ../mapping/pmap

DISPLAY = 1;

powerbot('connect','127.0.0.1',7001);

% Mapping
logFile = fopen('map.log', 'wt');
crStr = sprintf('\n');
odom = powerbot('getodom');

%log odometry
odomStr = sprintf('o %g %g %g\n',odom);
fwrite(logFile,odomStr,'char');

%performSlam('start',odom);

% Now enter the main behaviour loop. 

tic

%for i=1:3
while(toc<15*60)

    %map=performMapping(tilt0,disp,objects,configFile,is16BitDisp,logFile);

    if 0
        figure(10);
        hold on;
        % Plot odometry
        plot(map.odom(1),map.odom(2),'g+');
        % Plot laser
        lSize = size(map.laserPts);
        for i=1:lSize(1)
          plot(map.laserPts(i,1),map.laserPts(i,2),'r');
        end;
        % Plot object centers
        oSize = size(map.objPts);
        for i=1:oSize(1)
          plot(map.objPts(i,1),map.objPts(i,2),'b.'); % plot object center
        end;
        drawnow;
    end;
      
    % Move to a new location
    powerbot('movebot');

    iter = 1;
    speeds = [0 0 0];
    while iter <= 10 | abs(speeds(1)) > 0.01 | abs(speeds(3)) > 0.01
      odom = powerbot('getodom');
      laser = powerbot('getlaser');
 
      %log odometry
      odomStr = sprintf('o %g %g %g\n',odom);
      fwrite(logFile,odomStr,'char');

      %log laser
      fwrite(logFile,'l ','char');
      if length(laser) > 0
        laserStr = sprintf('%g ',laser);
        fwrite(logFile,laserStr,'char');
      end;
      fwrite(logFile,crStr,'char');

      if length(odom) == 3 & length(laser) == 361
        %performSlam('do_slam', odom, laser);
      end;

      if mod(iter, 10) == 0
        %performSlam('get_pmap');
      end;

      speeds = powerbot('getspeeds');
      iter = iter + 1;
    end;

    if DISPLAY
      figure(10);
      imshow(imread('coarse.pgm'));
    end;
end;

toc

%close log file
fclose(logFile);

%stop SLAM
%performSlam('get_rmap');
%performSlam('stop');

% if DISPLAY
%   figure(10);
%   imshow(imread('fine.pgm'));
% end;

powerbot('disconnect');

