%
% Generate bitmaps showing spectral saliency
% on int,r-g,y-b followed by MSER
%
addpath ../main/
addpath ~/matlab/drawing

SALIENCY_TOOLBOX = '../../resources/SaliencyToolbox';
addpath(genpath(SALIENCY_TOOLBOX));

dataset=2; % Use this dataset 

switch(dataset),
 case 1,
  fpath='/lci/project/raid1/srvc/x209_objects/'; % 320x240 images
  rl=[4];[3];
 case 2,
  fpath='/lci/project/raid1/srvc/bumblebee1/';  % 1024x768 images
  rl=[6];[4 5 6]; % Resolutions to try
 case 3,
  fpath='/lci/project/raid1/srvc/Hyatt_scouting/';  % 1600x1200 images
  rl=[6];
end

Nr=length(rl);

% Projections to try
plist=zeros(3,3); Np=size(plist,1);
plist(1,:)=[1 1 1]/3;  % grey
plist(2,:)=[1 -1 0];   % r-g
plist(3,:)=[1 1 -2]/2; % y-b

% Filters for spectral saliency
favg=fspecial('average', 3); % Fourier domain smoothing
savg=fspecial('disk', 3);   % Spatial smoothing

% MSER parameters
p.min_margin=10;
p.max_area=0.25;
p.min_size=10;30;
p.export_image=0; % Output greyscale images or not.

figure(1);clf

switch(dataset)
 case 1,
  fname='all.png';
 case 2,
  fname='frame01_left.png';
 case 3,
  fname='IMG_0145.JPG';
end


fprintf('loading image %s\n',fname);
  
inImg = imread([fpath fname]);
inImg = im2double(inImg);

fprintf('Building pyramid...');%tic;%Commented because there is now a tic in SRVC
imp=image_pyramid2(inImg,max(rl));
t1=toc;
fprintf('took %s\n',ctime(t1));

subplot(1,2,1);
image(inImg);axis image

fprintf('Saliency...');
[Iml,Sml,Tml]=spectral_saliency(imp,plist,rl,favg,savg);
t2=sum(Tml(:));
Sms=cell(Nr,1);  % Summed maps
for k=1:Nr,
  Sms{k}=(Sml{k,1}+Sml{k,2}+Sml{k,3})/3;
end
fprintf('took %s\n',ctime(t2));

fprintf('MSER...');tic
% Run MSER to find salient objects
%img=uint8(Sms{3}*255);
img=uint8(Sms{1}*255);
imean=mean(img(:));
[mvec,pvec,masks]=mser_objects2(img,p);

mvec2=mvec_upsample(mvec,rl(end));
keepl=find(pvec(3,:)>2.0*imean);   % Threshold on min(region)
%keepl=find(pvec(1,:)>2.0*imean);  % Threshold on mean(region)
bbox=bbox_from_conic(mvec(:,keepl));
keepl2=suppress_duplicates(bbox,0.5); % BB overlap (Better)
%keepl2=suppress_duplicates2(masks(keepl),.5); % 50% overlap

% Keep only needed masks
masks=masks(keepl(keepl2));
% Upsample masks
for k=1:length(masks)
  for l=1:rl-1,
    masks{k}=up2(masks{k});
  end
end
t3=toc;
fprintf('took %s\n',ctime(t3));
fprintf('Total time %s\n',ctime(t1+t2+t3));

% Draw MSERs on saliency map
subplot(1,2,2);
imagesc(img);axis image
hold on
h=plot_moments(mvec,1,'r');
h=plot_moments(mvec(:,keepl),1,'g');
  
% Draw MSERs on input image
subplot(1,2,1);
hold on
h=plot_moments(mvec2,1,'r');
h=plot_moments(mvec2(:,keepl(keepl2)),1,'g');


%
% Save figures as bitmaps
%
if 0,
  imwrite(uint8(inImg*255),'figure_input.png');
  
  saliencyImg=double(img)/255;
  intImg=Iml{1};
  rgImg=Iml{2};
  ybImg=Iml{3};
  for l=1:rl-1,
    saliencyImg=up2(saliencyImg);
    intImg=up2(intImg);
    rgImg=up2(rgImg);
    ybImg=up2(ybImg);
  end
  saliencyImg=repmat(saliencyImg,[1 1 3]);
  
  imwrite(uint8(intImg*255),'figure_int.png');
  imwrite(uint8(rgImg*255),'figure_rg.png');
  imwrite(uint8(ybImg*255),'figure_yb.png');

  imwrite(uint8(saliencyImg*255),'figure_saliency.png');
  
  mask=zeros(size(saliencyImg(:,:,1)));
  mask=draw_moments(mask,mvec2,3);
  mserImg=paint_mask(saliencyImg,mask,[1 0 0]);
  mask=zeros(size(saliencyImg(:,:,1)));
  mask=draw_moments(mask,mvec2(:,keepl(keepl2)),3);
  mserImg=paint_mask(mserImg,mask,[0 1 0]);

  imwrite(uint8(mserImg*255),'figure_mser.png');

  objectImg=paint_mask(inImg,mask,[0 1 0]);
  
  imwrite(uint8(objectImg*255),'figure_objects.png');
  
end

