%
% Generate bitmaps showing depth from stereo
%
addpath ../main/
addpath ../stereoCode/
addpath ~/matlab/drawing
addpath ../matlab_powerbot/
addpath ../../resources/PEImageGrabbing/
addpath ../stereoCode/

cameraPars.configFile = '../stereoCode/bb_id5020143.config';
cameraPars.canonfl=0;   % Use Canon camera if set, otherwise use Bumblebee

if cameraPars.canonfl
  cameraPars.CAMERAHEIGHT = 1.42;
  cameraPars.MAX_LOOK_DIST = 8;
else
  cameraPars.CAMERAHEIGHT = 1.38;
  cameraPars.MAX_LOOK_DIST = 3; % Only look if object is within 3m of camera
end;

% Using 16bit or 8bit disparity?
cameraPars.is16BitDisp = 1;

configPars.bpattern=0;  % For BB-HICOL-40
configPars.position_offset = 0.1; % position offset between camera and odometry
configPars.tilt_offset = 1; % tilt offset of 5 degrees, determined from PTU calibration

configPars.DISPLAY = 1;

if 0,
  fpath='/lci/project/raid1/srvc/bumblebee1/';  % 1024x768 images

  fno=10;
  fnamel=sprintf('frame%2d_left.png',fno);
  fnamer=sprintf('frame%2d_right.png',fno);
  fnamed=sprintf('frame%2d_depth.png',fno);


  fprintf('loading image %s\n',fname);

  il = imread([fpath fnamel]);
  il=colour_rectify(il,cameraPars.configFile,1);
  ir = imread([fpath fnamel]);
  ir=colour_rectify(ir,cameraPars.configFile,0);
  id = imread([fpath fnamed]);
else

  powerbot('connect','bart.ubcrobots.net',7001);
  state=powerbot('setpos',0,-45);
  
  buf=powerbot('getframe');

  bpattern=0;
  [il,ir]=bayer2rgbstereo(buf,0,bpattern,2);

  il_uint = uint8(il*255);
  ir_uint = uint8(ir*255);
    
  % Do stereo processing  
  im=uint8(zeros(size(il)));
  im(:,:,1)=il_uint(:,:,2);
  im(:,:,2)=ir_uint(:,:,2);
  if cameraPars.is16BitDisp
    disp=triclops_stereo16(im,cameraPars.configFile);
  else
    disp=triclops_stereo(im,cameraPars.configFile);
  end;

  [rows,cols]=size(disp);
    
  il = imresize( il_uint, [rows cols], 'bicubic' );
  ir = imresize( ir_uint, [rows cols], 'bicubic' );

  il=colour_rectify(il,cameraPars.configFile,1);
  ir=colour_rectify(ir,cameraPars.configFile,0); % Right field=0

  state=powerbot('getpos');
  tilt=state(2);
  objectImage = segmentGround( il, disp, tilt, cameraPars.is16BitDisp );    

  [objectImage2 obstacles]= segmentGround2(il,disp,tilt,cameraPars,configPars);

  if cameraPars.is16BitDisp
    id=disp;id(id>64257)=0;id=uint8(8/256*id);
  else
    id=disp;id(id>251)=0;id=8*id;
  end;
  io=uint8(objectImage*255);
  
  powerbot('disconnect');
end

subplot(2,2,1);
image(il);axis image

subplot(2,2,2);
image(ir);axis image

subplot(2,2,3);
image(id);axis image

subplot(2,2,4);
image(io);axis image

%imm1=repmat(il(:,:,2),[1 1 3]);
%imm2=repmat(il(:,:,2),[1 1 3]);
imm1=il;
imm2=il;
omask1=uint8(objectImage/max(objectImage(:))*255);
omask2=uint8(uint8(objectImage2)/uint8(max(objectImage2(:)))*255);
%imm1(:,:,2)=max(imm1(:,:,2),omask1);
imm1(:,:,2)=min(imm1(:,:,2),255-omask1);
%imm2(:,:,2)=max(imm2(:,:,2),omask2);
imm2(:,:,2)=min(imm2(:,:,2),255-omask2);
subplot(1,2,1);image(imm1);axis image
subplot(1,2,2);image(imm2);axis image

fprintf('Saving images...');
  
imwrite(ir,'figure_left.png');
imwrite(il,'figure_right.png');
imwrite(id,'figure_disp.png');
imwrite(io,'figure_omask.png');
imwrite(imm1,'figure_omask2.png');
imwrite(imm2,'figure_omask3.png');

fprintf('done.\n');

