%function imp=image_pyramid2(im,levels)
%
% Build a one octave pyramid from a
% colour image.
% Version with proper border treatment.
%
% IM     Input image RxCxD
% LEVELS Number of levels (default 6)
%
%Per-Erik Forssen, July 2007
function imp=image_pyramid2(im,levels)

if nargin<2,levels=6;end

% Blurring filter parameters
gstd=1;
supp=3;

gf=exp(-.5*([-supp:supp]/gstd).^2);gf=gf/sum(gf);

imp=cell(levels,1);

[rows,cols,ndim]=size(im);

imp{1}=im;  % First scale is input image

for k=2:levels
  rows=ceil(rows/2);
  cols=ceil(cols/2);
  imp{k}=zeros(rows,cols,ndim);
  % Build normalisation mask
  nm=ones(size(imp{k-1}(:,:,1)));
  nm=conv2(nm,gf,'same');
  nm=conv2(nm(:,1:2:end),gf','same');
  nm=nm(1:2:end,:);
  
  for m=1:ndim,
    bblr=conv2(double(imp{k-1}(:,:,m)),gf,'same');
    bblr=conv2(bblr(:,1:2:end),gf','same');
    %imp{k}(:,:,m)=max(0,min(1,bblr(1:2:end,:)));
    imp{k}(:,:,m)=bblr(1:2:end,:)./nm;
  end
end
