%function [mvec,pvec,masks]=mser_objects2(img,p)
%
%
% MVEC    Moment vector list 6xN
% PVEC    Property list (mean,max,min) 3xN
% MASKS   Region masks (binary 0/1)
%
%Per-Erik Forssen, July 2007
function [mvec,pvec,masks]=mser_objects2(img,p)

mser=extrema(img,p,[2]);   % Just MSER+
mser=mser{1,1};  % Unpack

Nbl=size(mser,2);

mvec=zeros(6,Nbl);
pvec=zeros(3,Nbl);
pvec(3,:)=255;

masks=cell(Nbl,1);
[rows,cols]=size(img);

for k=1:Nbl,
  r=mser{1,k};  % Pick a region
  masks{k}=uint8(zeros(rows,cols));
  for l=1:size(r,2),
    xl=r(2,l)+1;
    xh=r(3,l)+1;
    y=r(1,l)+1;
    masks{k}(y,xl:xh)=1;
    a=xh-xl+1;       % Area
    mvec(1,k)=mvec(1,k)+a;    
    mvec(2,k)=mvec(2,k)+0.5*(xh+xl)*a;
    mvec(3,k)=mvec(3,k)+y*a;
    mvec(4,k)=mvec(4,k)+xh*(xh+1)*(2*xh+1)/6-xl*(2*xl-1)*(xl-1)/6;
    mvec(5,k)=mvec(5,k)+0.5*y*(xh+xl)*a;
    mvec(6,k)=mvec(6,k)+y^2*a;
    iline=img(y,xl:xh);
    pvec(1,k)=pvec(1,k)+sum(iline);
    pvec(2,k)=max([pvec(2,k) iline]);
    pvec(3,k)=min([pvec(3,k) iline]);
  end
end

% Normalise and subtract centroid
for l=2:6,
  mvec(l,:)=mvec(l,:)./mvec(1,:);
end
mvec(4,:)=mvec(4,:)-mvec(2,:).^2;
mvec(5,:)=mvec(5,:)-mvec(2,:).*mvec(3,:);
mvec(6,:)=mvec(6,:)-mvec(3,:).^2;

% Normalise
pvec(1,:)=pvec(1,:)./mvec(1,:);
