%function [h = ] plot_ellipse(m,I,lw,col)
%
% Visualise a distribution as an ellipse
%
% M    Centroid
% I    Inertia matrix
% LW   Line width (default 0.5)
% COL  Colour string or RGB triplet
%      e.g. 'g' or [1 0 0]  (default 'r')
%
%Per-Erik Forssen, Sept 2002

function h=plot_ellipse(m,I,lw,col)

if nargin<3,lw=0.5;end
if nargin<4,col='r';end

[E,D]=eig(I);    % I=E*D*E' => rot'=E or rot'=[0 1;1 0]*E
R=E*2*sqrtm(D);

phil=0:pi/150:2*pi;
x=[cos(phil);sin(phil)];
Xp=R(1,:)*x;
Yp=R(2,:)*x;

if isa(col,'char')
  h=plot(m(1)+Xp,m(2)+Yp,col);
end
if prod(size(col))==3,
  h=plot(m(1)+Xp,m(2)+Yp);
  set(h,'Color',col);
end

set(h,'LineWidth',lw)
if(nargout==0),clear h;end