%
% spectral saliency measure in three scales
% and on int,r-g,y-b
% Significant clusters are then found by MSER
%
addpath ../main/

SALIENCY_TOOLBOX = '../../resources/SaliencyToolbox';
addpath(genpath(SALIENCY_TOOLBOX));

%fpath='/lci/project/raid1/srvc/DR2/robotImages/';
%fname='1.jpg';
fpath='/lci/project/raid1/srvc/bumblebee1/';
fname='frame01_left.png';
fname='frame15_left.png';
fname='frame17_left.png';


inImg = imread([fpath fname]);
inImg = im2double(inImg);

fprintf('Building pyramid...');tic;
imp=image_pyramid2(inImg);
t1=toc;
fprintf('took %s\n',ctime(t1));

% Resolutions to try
Nr=3;
resl=zeros(Nr,3);rl=[4 5 6];
resl(1,:)=size(imp{rl(1)}); % [96 128];
resl(2,:)=size(imp{rl(2)}); % [48 64];
resl(3,:)=size(imp{rl(3)}); % [24 32];

% Projections to try
plist=zeros(3,3); Np=size(plist,1);
plist(1,:)=[1 1 1]/3;  % grey
plist(2,:)=[1 -1 0];   % r-g
plist(3,:)=[1 1 -2]/2; % y-b

pnames=cell(3,1);
pnames{1}='intensity';
pnames{2}='r-g';
pnames{3}='y-b';

figure(3);clf
image(inImg);axis image

favg=fspecial('average', 3); % Fourier domain smoothing
savg=fspecial('disk', 3);   % Spatial smoothing

fprintf('Saliency...');
[Iml,Sml,Tml]=spectral_saliency(imp,plist,rl,favg,savg);
t2=sum(Tml(:));
fprintf('took %s\n',ctime(t2));

figure(1);
for l=1:Np,
  for k=1:Nr,
    subplot(Np,Nr,k+(l-1)*Nr);
    imshow(Iml{k,l},[min(Iml{k,l}(:)) max(Iml{k,l}(:))]);
    title(sprintf('%dx%d',resl(k,1),resl(k,2)));
    ylabel(pnames{l});
  end
end
figure(2);
for l=1:Np,
  for k=1:Nr,
    subplot(Np+1,Nr,k+(l-1)*Nr);imshow(Sml{k,l});
    title(sprintf('%s seconds',ctime(Tml(k,l))));
    ylabel(pnames{l});
  end
end
Sms=cell(Nr,1);  % Summed maps
for k=1:Nr,
  Sms{k}=(Sml{k,1}+Sml{k,2}+Sml{k,3})/3;
  subplot(Np+1,Nr,Np*Nr+k);imshow(Sms{k});
  title('Sum');
end

if 0,
  % Comparison with Walther's toolbox
  inputImage=zeros(240,320,3);
  for k=1:3,
    inputImage(:,:,k) = imresize(inImg(:,:,k), [240 320], 'bilinear');
  end
  inputImage=max(0,min(1,inputImage));
  objectMask=ones(240,320);
  K=4;  % Number of objects to find
  tic;[ objects ] = getObjects( inputImage, objectMask, K);toc

  % Display results
  figure(3);clf
  subplot(2,1,1);
  imagesc(inputImage);axis image
  title('Input image');
  for k=1:K,
    subplot(4,2,4+k);
    imagesc(objects(k).shapeData.shapeMap.data);
    axis image
    title(['Object ' num2str(k)]);
  end
end

if 1,
  % Run MSER to find salient objects
  p.min_margin=10;
  p.max_area=0.25;
  p.min_size=10;30;
  p.export_image=0; % Output greyscale images or not.
  img=uint8(Sms{3}*255);
  imean=mean(img(:));
  [mvec,pvec]=mser_objects(img,p);
  mvec2=mvec_upsample(mvec,6);
  %keepl=pvec(3,:)>1.5*imean;
  keepl=pvec(1,:)>2.0*imean;
  
  figure(3);clf
  subplot(2,1,1);imagesc(img);axis image
  hold on
  h=plot_moments(mvec,1,'r');
  h=plot_moments(mvec(:,keepl),1,'g');
  
  subplot(2,1,2);image(inImg);axis image
  hold on
  h=plot_moments(mvec2,1,'r');
  h=plot_moments(mvec2(:,keepl),1,'g');
end