%
% Spectral saliency on int,r-g,y-b followed by MSER
% Show results for 5 images at a time
%
addpath ../main/

SALIENCY_TOOLBOX = '../../resources/SaliencyToolbox';
addpath(genpath(SALIENCY_TOOLBOX));

dataset=2; % Use this dataset 

switch(dataset),
 case 1,
  fpath='/lci/project/raid1/srvc/x209_objects/'; % 320x240 images
  rl=[4];[3];
 case 2,
  fpath='/lci/project/raid1/srvc/bumblebee1/';  % 1024x768 images
  rl=[6];[4 5 6]; % Resolutions to try
 case 3,
  fpath='/lci/project/raid1/srvc/Hyatt_scouting/';  % 1600x1200 images
  rl=[6];
end

Nr=length(rl);

% Projections to try
plist=zeros(3,3); Np=size(plist,1);
plist(1,:)=[1 1 1]/3;  % grey
plist(2,:)=[1 -1 0];   % r-g
plist(3,:)=[1 1 -2]/2; % y-b

% Filters for spectral saliency
favg=fspecial('average', 3); % Fourier domain smoothing
savg=fspecial('disk', 3);   % Spatial smoothing

% MSER parameters
p.min_margin=10;
p.max_area=0.25;
p.min_size=10;30;
p.export_image=0; % Output greyscale images or not.

figure(1);clf

switch(dataset)
 case 1,
  Nfrm=5;
  fname=cell(Nfrm,1);
  fname{1}='all.png';
  %fname{2}='all_far.png';
  %fname{3}='standing.png';
  fname{2}='stool_close.png';
  fname{3}='table_3.png';
  fname{4}='table_4.png';
  fname{5}='table_5.png';
 case 2,
  % Pick a subset of 5 images:
  fnol=[1:5];
  %fnol=[6:10];
  %fnol=[11:15];
  %fnol=[16:20];
  %fnol=[21:25];
  %fnol=[26:31];
  Nfrm=length(fnol);
  fname=cell(Nfrm,1);
  for f=1:Nfrm,
    fname{f}=sprintf('frame%02d_left.png',fnol(f));
  end
 case 3,
  fnol=[133:136];
  fnol=[137:140];
  fnol=[141:144];
  fnol=[145:148];
  %fnol=[149:152];
  Nfrm=length(fnol);
  fname=cell(Nfrm,1);
  for f=1:Nfrm,
    fname{f}=sprintf('IMG_%04d.JPG',fnol(f));
  end
end

for f=1:Nfrm,
  fprintf('loading image %s\n',fname{f});
  
  inImg = imread([fpath fname{f}]);
  inImg = im2double(inImg);

  fprintf('Building pyramid...');tic;
  imp=image_pyramid2(inImg,max(rl));
  t1=toc;
  fprintf('took %s\n',ctime(t1));

  subplot(2,Nfrm,f);
  image(inImg);axis image

  fprintf('Saliency...');
  [Iml,Sml,Tml]=spectral_saliency(imp,plist,rl,favg,savg);
  t2=sum(Tml(:));
  Sms=cell(Nr,1);  % Summed maps
  for k=1:Nr,
    Sms{k}=(Sml{k,1}+Sml{k,2}+Sml{k,3})/3;
  end
  fprintf('took %s\n',ctime(t2));

  fprintf('MSER...');tic
  % Run MSER to find salient objects
  %img=uint8(Sms{3}*255);
  img=uint8(Sms{1}*255);
  imean=mean(img(:));
  [mvec,pvec,masks]=mser_objects2(img,p);
  
  mvec2=mvec_upsample(mvec,rl(end));
  keepl=find(pvec(3,:)>2.0*imean);   % Threshold on min(region)
  %keepl=find(pvec(1,:)>2.0*imean);  % Threshold on mean(region)
  bbox=bbox_from_conic(mvec(:,keepl));
  keepl2=suppress_duplicates(bbox,0.5); % BB overlap (Better)
  %keepl2=suppress_duplicates2(masks(keepl),.5); % 50% overlap

  % Keep only needed masks
  masks=masks(keepl(keepl2));
  % Upsample masks
  for k=1:length(masks)
    for l=1:rl-1,
      masks{k}=up2(masks{k});
    end
  end
  t3=toc;
  fprintf('took %s\n',ctime(t3));
  fprintf('Total time %s\n',ctime(t1+t2+t3));
    
  % Draw MSERs on saliency map
  subplot(2,Nfrm,Nfrm+f);
  imagesc(img);axis image
  hold on
  h=plot_moments(mvec,1,'r');
  h=plot_moments(mvec(:,keepl),1,'g');
  
  % Draw MSERs on input image
  subplot(2,Nfrm,f);
  hold on
  h=plot_moments(mvec2,1,'r');
  h=plot_moments(mvec2(:,keepl(keepl2)),1,'g');
end

if 0,
  figure(2);clf
  Nobj=length(masks);
  for k=1:Nobj,
    subplot(3,3,k);
    img=inImg;
    imgr=mean(inImg,3)*.25+.5;
    img(:,:,1)=img(:,:,1).*masks{k}+imgr.*(1-masks{k});
    img(:,:,2)=img(:,:,2).*masks{k}+imgr.*(1-masks{k});
    img(:,:,3)=img(:,:,3).*masks{k}+imgr.*(1-masks{k});
    image(img);axis image
  end
end