%function keepl=suppress_duplicates(bbox,othr)
%
% Suppress regions with large overlap, i.e.,
%
%    intersect(A,B)/union(A,B)>othr
%
% If an overlap is too large, the larger of
% the two regions is kept.
%
% BBOX  Bounding box list
% OTHR  Overlap threshold (default 0.5)
%
% Note: Actually bounding boxes instead of
% ellipses are used for intersection computation.
%
%Per-Erik Forssen, July 2007
function keepl=suppress_duplicates(bbox,othr)

if nargin<2,othr=0.5;end

% Compute all areas
a=(bbox(3,:)-bbox(1,:)).*(bbox(4,:)-bbox(2,:));

% Sort with ascending area
[dummy,indl]=sort(a);
a=a(indl);
bbox=bbox(:,indl);

Nb=size(bbox,2);

keepl=ones(Nb,1);

for k=1:Nb-1,
  for l=k+1:Nb,
    ymin=max(bbox(1,[k l]));
    xmin=max(bbox(2,[k l]));
    ymax=min(bbox(3,[k l]));
    xmax=min(bbox(4,[k l]));
    a_intersect=0;
    if(ymax>ymin)
      if(xmax>xmin)
        a_intersect=(ymax-ymin)*(xmax-xmin);
      end
    end
    a_union=a(k)+a(l)-a_intersect;
    if(a_intersect/a_union>othr)
      keepl(k)=0;
      break;
    end
    %fprintf('overlap=%g\n',a_intersect/a_union);
  end
end

keepl=indl(find(keepl));
