%function keepl=suppress_duplicates2(masks,othr)
%
% Suppress regions with large overlap, i.e.,
%
%    intersect(A,B)/union(A,B)>othr
%
% If an overlap is too large, the larger of
% the two regions is kept.
%
% MASKS Cell array with region masks
% OTHR  Overlap threshold (default 0.5)
%
% Note: Actually bounding boxes instead of
% ellipses are used for intersection computation.
%
%Per-Erik Forssen, July 2007
function keepl=suppress_duplicates2(masks,othr)

if nargin<2,othr=0.5;end

Nb=length(masks);

% Compute all areas
a=zeros(Nb,1);
for k=1:Nb,
  a(k)=sum(masks{k}(:));
end

% Sort with ascending area
[dummy,indl]=sort(a);
a=a(indl);
masks=masks(indl);

keepl=ones(Nb,1);

for k=1:Nb-1,
  for l=k+1:Nb,
    a_intersect=sum(sum(min(masks{k},masks{l})));
    a_union=a(k)+a(l)-a_intersect;
    if(a_intersect/a_union>othr)
      keepl(k)=0;
      break;
    end
    %fprintf('overlap=%g\n',a_intersect/a_union);
  end
end

keepl=indl(find(keepl));
