/*
 * Read triclops parameters and a stereo image from file,
 * compute stereo and write to file.
 *
 * Per-Erik Forssen, Feb 2007
 */
//===========================================================================
// System Includes
//===========================================================================
#include <stdio.h>
#include <stdlib.h>
#include <sys/timeb.h>
#include <time.h>

//===========================================================================
// Point Grey Research Includes
//===========================================================================
#include "triclops.h"
#include "pnmutils.h"

int main( int argc, char** argv ) {
  TriclopsContext   triclops;
  TriclopsError     te;
  //char config_filename[] = "bbhicol40.config";
  //char config_filename[] = "bb_id2210020.config";
  char config_filename[] = "bb_id5020143.config";
  
  // extract Triclops context from file
  te =  triclopsGetDefaultContextFromFile( &triclops, config_filename);
  if ( te != TriclopsErrorOk ) {
    printf( "TriclopsError: GetDefaultContextFromFile : %s\n", 
	    triclopsErrorToString( te ) );
    return 1;
  }

  TriclopsInput  triclopsInput;
  ppmReadToTriclopsInputRGB( "stereo.ppm", &triclopsInput );

  te = triclopsPreprocess( triclops, &triclopsInput );
  if ( te != TriclopsErrorOk ) {
    printf( "TriclopsError: triclopsPreprocess : %s\n", 
	    triclopsErrorToString( te ) );
    return 1;
  }

  te = triclopsStereo( triclops );
  if ( te != TriclopsErrorOk ) {
    printf( "TriclopsError: triclopsStereo : %s\n", 
	    triclopsErrorToString( te ) );
    return 1;
  }

  TriclopsImage  triclopsImage;
  triclopsGetImage( triclops, TriImg_DISPARITY, TriCam_REFERENCE,
		    &triclopsImage );
  triclopsSaveImage( &triclopsImage, "disp.pgm" );
  
  printf("Hello\n");

  return 0;
}
