/*
 * Write triclops parameters to file to allow
 * later offline processing.
 * Per-Erik Forssen, Feb 2007
 */
//===========================================================================
// System Includes
//===========================================================================
#include <stdio.h>
#include <stdlib.h>
#include <sys/timeb.h>
#include <time.h>

//===========================================================================
// Point Grey Research Includes
//===========================================================================
#include "digiclops.h"
#include "triclops.h"


#define _HANDLE_DIGICLOPS_ERROR( functionName ) \
{ \
   if( de != DIGICLOPS_ok ) \
   { \
      printf( \
         "Digiclops Error in %s: %s\n", \
	 functionName, \
	 ::digiclopsErrorToString( de ) ); \
      return -1; \
   }  \
} \
\


int main( int argc, char** argv ) {
  DigiclopsContext  digiclops;
  DigiclopsError    de;
  TriclopsContext   triclops;
  TriclopsError     te;
  char config_filename[] = "bbhicol40.config";
  
  de  = digiclopsCreateContext( &digiclops );
  _HANDLE_DIGICLOPS_ERROR( "digiclopsCreateContext()" );
  
  // initialize Digiclops device 0
  de  = digiclopsInitialize( digiclops, 0 );
  _HANDLE_DIGICLOPS_ERROR( "digiclopsInitialize()" );
  
  // extract Triclops context from camera
  de = digiclopsGetTriclopsContextFromCamera( digiclops, &triclops );
  _HANDLE_DIGICLOPS_ERROR( "digiclopsGetTriclopsContextFromCamera()" );
  
  te = triclopsWriteDefaultContextToFile(triclops, config_filename);
  if ( te != TriclopsErrorOk ) {
    printf( "TriclopsError: WriteDefaultContextToFile : %s\n", 
	    triclopsErrorToString( te ) );
    return 1;
  }
  
  printf("Wrote config file: %s\n",config_filename);
  de  = digiclopsDestroyContext( digiclops );
  _HANDLE_DIGICLOPS_ERROR( "digiclopsDestroyContext()" );
  
  return 0;
}
