/*
**  camera.c
**  Matlab MEX interface to libdc1394 for
**  stereo image acquisition.
** /cs/public/src/libdc1394-1.1.0/installdir/libdc1394/dc1394_control.h
*/

#include "mex.h"
#include "mexutil.h"
#include <libraw1394/raw1394.h>
#include <libdc1394/dc1394_control.h>
#include <string.h>
#include "string1394.h"
#include <time.h>

#define MAX_PORTS 4
#define MAX_RESETS 10
#define DMA_DEV_NAME "/dev/video1394-0"
/* #define DMA_DEV_NAME "/dev/video1394" */
#define DROP_FRAMES 1
#define DC1394_BUFFERS 8

u_int64_t g_guid = 0; /* guid for camera, or 0 for first camera on bus */
int connectfl = 0;  /* Connection state */
int capturefl = 0;  /* Capture prepared state */
int interlacedfl=1;  /* Remember interlaced state */
/* camera handle and camera struct */
raw1394handle_t handle;
dc1394_cameracapture camera;

void write_time_diff(char *strg,clock_t t0,clock_t t1) {
 printf("%s %g\n",strg,(double)(t1-t0)/CLOCKS_PER_SEC);
}

int camera_connect(char *dma_dev_name,int infofl) {
 /* dc1394 variables */
 dc1394_camerainfo info;
 dc1394_feature_set features;
 dc1394_feature_info feature;
 struct raw1394_portinfo ports[MAX_PORTS];
 int numPorts = 0;
 int numCameras = 0;
 int i,j;
 nodeid_t * camera_nodes = NULL;
 int found = 0;

 if(infofl) mexPrintf("Connecting to device file %s\n",dma_dev_name);

  /* Open ohci and asign handle to it */
  handle = raw1394_new_handle();
  if (handle==NULL) {
    mexErrMsgTxt("Unable to aquire a raw1394 handle\n\n"
		 "Please check \n"
		 "  - if the kernel modules `ieee1394',`raw1394' and `ohci1394' are loaded \n"
		 "  - if you have read/write access to /dev/raw1394\n\n");
    return 0;
  }

  /* get the number of ports (cards) */
  numPorts = raw1394_get_port_info(handle, ports, numPorts);
  /*   mexPrintf("numPorts=%d\n",numPorts); */
  raw1394_destroy_handle(handle);
  handle = NULL;

  for (j = 0; j < MAX_RESETS && found == 0; j++) {
    /* look across all ports for cameras */
    for (i = 0; i < numPorts && found == 0; i++) {
      if (handle != NULL)
	dc1394_destroy_handle(handle);
      handle = dc1394_create_handle(i);
      if (handle == NULL) {
	fprintf( stderr, "Unable to acquire a raw1394 handle for port %i\n", i);
	return 0;
      }
      numCameras = 0;
      camera_nodes = dc1394_get_camera_nodes(handle, &numCameras, 0);
      if (numCameras > 0) {
	if (g_guid == 0) {
	  /* use the first camera found */
	  camera.node = camera_nodes[0];
	  if (dc1394_get_camera_info(handle, camera_nodes[0], &info) == DC1394_SUCCESS)
	    if(infofl) dc1394_print_camera_info(&info);
	  found = 1;
	} else {
	  /* attempt to locate camera by guid */
	  int k;
	  for (k = 0; k < numCameras && found == 0; k++) {
	    if (dc1394_get_camera_info(handle, camera_nodes[k], &info) == DC1394_SUCCESS) {
	      if (info.euid_64 == g_guid) {
		if(infofl) dc1394_print_camera_info(&info);
		camera.node = camera_nodes[k];
		found = 1;
	      }
	    }
	  }
	}
	if (found == 1) {
	  /* camera can not be root--highest order node */
	  if (camera.node == raw1394_get_nodecount(handle)-1) {
	    /* reset and retry if root */
	    raw1394_reset_bus(handle);
	    sleep(2);
	    found = 0;
	  }
	}
	dc1394_free_camera_nodes(camera_nodes);
      } /* cameras >0 */
    } /* next port */
    /*     mexPrintf("Retry.\n"); */
  } /* next reset retry */

  if (found == 0 && g_guid != 0) {
    mexErrMsgTxt("Unable to locate camera node by guid\n");
    return 0;
  }
  else if (numCameras == 0) {
    mexErrMsgTxt("no cameras found :(\n");
    dc1394_destroy_handle(handle);
    return 0;
  }
  if (j == MAX_RESETS) {
    mexErrMsgTxt( "failed to not make camera root node :(\n");
    dc1394_destroy_handle(handle);
    return 0;
  }
  
  /*
   *   Read out feature set (a lot of info)
   */
  if(infofl>1) { 
    if(dc1394_get_camera_feature_set(handle,camera.node,&features)) {
      dc1394_print_feature_set(&features);
    }
  }

  return 1;  /* Success */
}

int camera_disconnect() {
  /* Close camera */
  dc1394_dma_unlisten(handle,&camera);
  if(capturefl)
    dc1394_dma_release_camera(handle,&camera);
  capturefl=0;
  dc1394_destroy_handle(handle);
  return 1;  /* Success */
}

int camera_setwhitebalance(int whitebal_ub,int whitebal_vr) {
  if(dc1394_set_white_balance(handle,camera.node,whitebal_ub,whitebal_vr)) {
    return 1;
  } else {
    mexPrintf("Failed to set whitebalance\n");
    return 0;
  }
}

int camera_getwhitebalance(int *whitebal_ub,int *whitebal_vr) {
  if(dc1394_get_white_balance(handle,camera.node,whitebal_ub,whitebal_vr)) {
    return 1;
  } else {
    mexPrintf("Failed to get whitebalance\n");
    return 0;
  }
}

int camera_setbrightness(int bval) {
  if(dc1394_set_brightness(handle,camera.node,bval)) {
    return 1;
  } else {
    mexPrintf("Failed to set brightness\n");
    return 0;
  }
}

int camera_getbrightness(int *bval) {
  if(dc1394_get_brightness(handle,camera.node,bval)) {
    return 1;
  } else {
    mexPrintf("Failed to get brightness\n");
    return 0;
  }
}

int camera_setshutter(int sval) {
  if(dc1394_set_shutter(handle,camera.node,sval)) {
    return 1;
  } else {
    mexPrintf("Failed to set shutter\n");
    return 0;
  }
}

int camera_getshutter(int *sval) {
  if(dc1394_get_shutter(handle,camera.node,sval)) {
    return 1;
  } else {
    mexPrintf("Failed to get shutter\n");
    return 0;
  }
}

int camera_setframerate(int sval) {
  if(dc1394_set_video_framerate(handle,camera.node,sval)) {
    return 1;
  } else {
    mexPrintf("Failed to set framerate\n");
    return 0;
  }
}

int camera_getframerate(int *sval) {
  if(dc1394_get_video_framerate(handle,camera.node,sval)) {
    return 1;
  } else {
    mexPrintf("Failed to get framerate\n");
    return 0;
  }
}

int camera_setvideomode(int sval) {
  if(dc1394_set_video_mode(handle,camera.node,sval)) {
    return 1;
  } else {
    mexPrintf("Failed to set video mode\n");
    return 0;
  }
}

int camera_getvideomode(int *sval) {
  if(dc1394_get_video_mode(handle,camera.node,sval)) {
    return 1;
  } else {
    mexPrintf("Failed to get video mode\n");
    return 0;
  }
}

void camera_printinfo(dc1394_miscinfo *misc_info) {

  mexPrintf("minfo.format %s\n",getstr_format(misc_info->format));
  mexPrintf("     .mode %s\n",getstr_mode(misc_info->mode));
  mexPrintf("     .framerate %s\n",getstr_framerate(misc_info->framerate));
  if(misc_info->is_iso_on) {
    mexPrintf("     .is_iso_on true\n");
    mexPrintf("     .iso_channel %d\n",misc_info->iso_channel);
    mexPrintf("     .iso_speed %s\n",getstr_speed(misc_info->iso_speed));
  } else {
    mexPrintf("     .is_iso_on false\n");
  }
}

int camera_getinfo() {
  dc1394_miscinfo misc_info;
  /* Read current settings */
  if(dc1394_get_camera_misc_info(handle,camera.node,&misc_info)!=DC1394_SUCCESS) {
    mexErrMsgTxt("Could not get camera misc info\n");
    return 0;
  }
  camera_printinfo(&misc_info);
  return 1; /* Success */
}

/*
**  mexFunction is the interface function to MATLAB.
**
** plhs - pointer to left-hand OUTPUT mxArrays
** nlhs - number of left-hand OUTPUT arrays
** prhs - pointer to right-hand INPUT mxArrays
** nrhs - number of right-hand INPUT arrays
**
*/
#define BUF_LEN 50
void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
 /* dc1394 variables */
 dc1394_miscinfo misc_info;
 char cstr[BUF_LEN+1];
 char cstr2[BUF_LEN+1];
 char *cstr3;
 char *dma_dev_name = DMA_DEV_NAME; 
 int x,y,d,rows,cols;
 /* Matlab variables */
 int out_dims[3];
 const int *arg_dims;
 int arg_ndims;
 int msize[]={1,1};       /* Matrix size of scalar */
 int infofl=0;       /* Default */
 unsigned char *matlab_bufp,*camera_bufp;
 int whitebal_ub,whitebal_vr;
 int bval,sval;
 clock_t t1,t2,t3,t4,t5,t6;

 /* Argument count check */
  if(nrhs ==0) goto mexErrExit;

  /* Parse arg 1 (request string) */
  arg_dims = mxGetDimensions(prhs[0]);
  arg_ndims = mxGetNumberOfDimensions(prhs[0]);
  if(mxGetClassID(prhs[0]) != mxCHAR_CLASS) {
    mexPrintf("Type mismatch: <request> should be of class char.\n");
    goto mexErrExit;
  }
  mxGetString(prhs[0], cstr,BUF_LEN);
  
  /* CONNECT */
  if(strcmp(cstr,"connect")==0) {
    if(connectfl) {
      mexErrMsgTxt("Already connected.");
      goto mexExit;
    } else {
      if(nrhs>=2) {
	/* Parse arg 1 */
	if(!type_check("infofl",prhs[1],mxDOUBLE_CLASS,2,msize))
	  goto mexErrExit;
	infofl=(int)mxGetScalar(prhs[1]);
      }
      if(camera_connect(DMA_DEV_NAME,infofl)) connectfl=1;
    }
  }

  /* All other commands require a connection */

  if(!connectfl) {
    mexErrMsgTxt("Not connected.");
    goto mexExit;
  }

  /* DISCONNECT */
  if(strcmp(cstr,"disconnect")==0) {
    if(camera_disconnect())
      connectfl=0;
  }

  /* SETUP_CAPTURE */
  if(strcmp(cstr,"setup_capture")==0) {
    if(capturefl==1) {
      mexErrMsgTxt("already setup");
      goto mexErrExit;
    }
    if(dc1394_get_camera_misc_info(handle,camera.node,&misc_info)!=DC1394_SUCCESS) {
      mexErrMsgTxt("Could not get camera misc info\n");
      return;
    }
    /* Setting for new bumblebee BB_HICOL_40 */
    /*   misc_info.format=FORMAT_SVGA_NONCOMPRESSED_1; */
    /*   misc_info.mode= MODE_1024x768_MONO16; */
    
    /* Setting for old bumblebee BB_COL_40 */
    /*   misc_info.format=FORMAT_VGA_NONCOMPRESSED; */
    /*   misc_info.mode=MODE_640x480_RGB;   */
    /*   misc_info.mode=MODE_640x480_MONO16;      */
    /*   misc_info.framerate=FRAMERATE_15; */

/*     misc_info.framerate=FRAMERATE_30; */

    if(nrhs>=2) {
      /* Parse arg 1 */
      if(!type_check("infofl",prhs[1],mxDOUBLE_CLASS,2,msize))
	goto mexErrExit;
      infofl=(int)mxGetScalar(prhs[1]);
    }
    if(infofl) camera_printinfo(&misc_info);

    /* Temporary fix for 1394b card */
/*     dc1394_set_operation_mode(handle, camera.node,OPERATION_MODE_1394B);  */
/*     dc1394_set_operation_mode(handle, camera.node,OPERATION_MODE_LEGACY); */
    
    if (dc1394_dma_setup_capture(handle, camera.node,
				 misc_info.iso_channel, /* channel */ 
				 misc_info.format,      /* format */
				 misc_info.mode,        /* video mode */
				 misc_info.iso_speed,   /* speed */
				 misc_info.framerate,   /* framerate */
				 DC1394_BUFFERS,
				 DROP_FRAMES, 
				 dma_dev_name,
				 &camera)!=DC1394_SUCCESS) {
      fprintf( stderr,"unable to setup camera-\n"
	       "check line %d of %s to make sure\n"
	       "that the video mode,framerate and format are\n"
	       "supported by your camera\n",
	       __LINE__,__FILE__);
      dc1394_dma_unlisten(handle,&camera);
      return;
    }
    capturefl=1;   /* setup_capture done */
   /* have the camera start sending us data */
    if (dc1394_start_iso_transmission(handle,camera.node)
	!=DC1394_SUCCESS) {
      mexErrMsgTxt("unable to start camera iso transmission\n");
      dc1394_dma_unlisten(handle,&camera);
      if(capturefl)
	dc1394_dma_release_camera(handle,&camera);
      capturefl=0;
      return;
    }    

    /* Set interlacedfl for getframe */
    interlacedfl=1;
    switch(misc_info.mode) {
    case MODE_1024x768_MONO16:
      break;
    case  MODE_640x480_MONO16:
      break;
    default:
      interlacedfl=0;
    }     
  }

  /* STOP_CAPTURE */
  if(strcmp(cstr,"stop_capture")==0) {
    /* Close dma channel to camera */
    dc1394_dma_unlisten(handle,&camera);  /* Stop iso */
    if(capturefl)
      dc1394_dma_release_camera(handle,&camera);  /* undo setup capture */
    capturefl=0;
  }

  /* GETFRAME */
  if(strcmp(cstr,"getframe")==0) {
    /* Inline code for getframe. Should move to fcn.... */
    if(capturefl==0) {
      mexErrMsgTxt("setup capture first");
      goto mexErrExit;
    }
     
    t1=clock();
    /* capture one frame */
    if (dc1394_dma_single_capture(&camera)!=DC1394_SUCCESS) {
      mexErrMsgTxt("unable to capture a frame\n");
      dc1394_dma_unlisten(handle,&camera);
      if(capturefl)
	dc1394_dma_release_camera(handle,&camera);
      capturefl=0;
      return;
    }

    t2=clock();
    if(0) {
      /* Allocate matlab array (Convert sensor array to 3 fields) */
      rows=camera.frame_height;
      cols=camera.frame_width;
      out_dims[0]=rows;
      out_dims[1]=cols;
      out_dims[2]=3;   
      plhs[0] = mxCreateNumericArray(3,out_dims,mxUINT8_CLASS,mxREAL);
      
      matlab_bufp = (unsigned char *)mxGetPr(plhs[0]);
      camera_bufp = (unsigned char *)camera.capture_buffer;
      
      t3=clock();
      
      if(interlacedfl) {
	/* Convert interlaced(mono16) to two fields */
	for(d=0;d<2;d++) {
	  for(x=0;x<cols;x++) {
	    for(y=0;y<rows;y++) {
	      matlab_bufp[y+x*rows+d*rows*cols]=camera_bufp[2*x+y*cols*2+d]; 
	    }
	  }
	}
      } else {
	/* Convert interlaced(rgb) to three fields */
	for(d=0;d<3;d++) {
	  for(x=0;x<cols;x++) {
	    for(y=0;y<rows;y++) {
	      matlab_bufp[y+x*rows+d*rows*cols]=camera_bufp[d+3*x+y*3*cols]; 
	    }
	  }
	}
      }
      t4=clock();
    } else {
      /* Allocate matlab array (Just output sensor array) */
      /* Extra fields go into width: */
      rows=camera.frame_height;
      cols=camera.quadlets_per_frame*4/camera.frame_height;
      /* dma_frame_size gives wrong size for YUV4xx formats */
/*       cols=camera.dma_frame_size/camera.frame_height;  */
      out_dims[0]=cols;
      out_dims[1]=rows;
      out_dims[2]=1;   
      plhs[0] = mxCreateNumericArray(3,out_dims,mxUINT8_CLASS,mxREAL);
      matlab_bufp = (unsigned char *)mxGetPr(plhs[0]);
      camera_bufp = (unsigned char *)camera.capture_buffer;
      
      t3=clock();
      memcpy(matlab_bufp,camera_bufp,rows*cols*sizeof(unsigned char));
      t4=clock();
    }
      /* Release buffer */
      dc1394_dma_done_with_buffer(&camera);
    t5=clock();
/*     write_time_diff("capture time",t1,t2); */
/*     write_time_diff("alloc time",t2,t3); */
/*     write_time_diff("copy time",t3,t4); */
/*     write_time_diff("release time",t4,t5); */
  }

  /* GETINFO */
  if(strcmp(cstr,"getinfo")==0) {
    camera_getinfo();
  }

  /* WHITEBALANCE */
  if(strcmp(cstr,"whitebalance")==0) {
    if(nrhs==3) {
      /* Parse arg 2 */
      if(!type_check("whitebal_ub",prhs[1],mxDOUBLE_CLASS,2,msize))
	goto mexErrExit;
      whitebal_ub=(int)mxGetScalar(prhs[1]);
      /* Parse arg 3 */
      if(!type_check("whitebal_vr",prhs[2],mxDOUBLE_CLASS,2,msize))
	goto mexErrExit;
      whitebal_vr=(int)mxGetScalar(prhs[2]);
      
      camera_setwhitebalance(whitebal_ub,whitebal_vr);
    }
    camera_getwhitebalance(&whitebal_ub,&whitebal_vr);
    if(nlhs>=1) plhs[0] = mxCreateDoubleScalar(whitebal_ub);
    if(nlhs>=2) plhs[1] = mxCreateDoubleScalar(whitebal_vr);
  }

  /* Add auto attributes for these */

  /* BRIGHTNESS */
  if(strcmp(cstr,"brightness")==0) {
    if(nrhs==2) {
      /* Parse arg 2 */
      if(!type_check("bval",prhs[1],mxDOUBLE_CLASS,2,msize))
	goto mexErrExit;
      bval=(int)mxGetScalar(prhs[1]);
      
      camera_setbrightness(bval);
    }
    camera_getbrightness(&bval);
    if(nlhs>=1) plhs[0] = mxCreateDoubleScalar(bval);
  }

  /* SHUTTER */
  if(strcmp(cstr,"shutter")==0) {
    if(nrhs==2) {
      /* Parse arg 2 */
      if(!type_check("sval",prhs[1],mxDOUBLE_CLASS,2,msize))
	goto mexErrExit;
      sval=(int)mxGetScalar(prhs[1]);
      
      camera_setshutter(sval);
    }
    camera_getshutter(&sval);
    if(nlhs>=1) plhs[0] = mxCreateDoubleScalar(sval);
  }

  /* FRAMERATE */
  if(strcmp(cstr,"framerate")==0) {
    if(nrhs==2) {
      /* Parse arg 2 */
      if(!type_check("framerate",prhs[1],mxDOUBLE_CLASS,2,msize))
	goto mexErrExit;
      /* Convert to string */
      snprintf(cstr2,BUF_LEN,"%g",mxGetScalar(prhs[1]));
      /* Convert to enum */
      sval=getval_framerate(cstr2);
      /* Set framerate */
      camera_setframerate(sval);
    }
    camera_getframerate(&sval);

    if(nlhs>=1)
      plhs[0] = mxCreateDoubleScalar(strtod(getstr_framerate(sval),NULL));
  }

  /* VIDEO_MODE */
  if(strcmp(cstr,"video_mode")==0) {
    if(nrhs==2) {
      /* Parse arg 2 */
      msize[1]=-1;
      if(!type_check("video_mode",prhs[1],mxCHAR_CLASS,2,msize))
	goto mexErrExit;
      /* Convert to string */
      cstr3=mxArrayToString(prhs[1]);
      /* Convert to enum */
      sval=getval_mode(cstr3);
      mxFree(cstr3);
      /* Set video mode */
      camera_setvideomode(sval);
    }
    camera_getvideomode(&sval);
    cstr3=getstr_mode(sval);

    if(nlhs>=1)
      plhs[0] = mxCreateCharMatrixFromStrings(1,(const char **)&cstr3);
  }

/* End of show :-) */

  goto mexExit;

/* Exit with error message */
mexErrExit:
 mexPrintf("Usage: <return_vals>=camera(<request>[,<params>,...])\n");
mexExit: {}
}
