%function [P1,P2]=clip_lines2(L,imbox)
%
%  Generate two lists of line endpoints
% defined by clipping the mathematical lines
% in the columns of L with a bounding box.
%
%  L     List of lines in homogeneous form (3xN)
%  IMBOX image bounding box [yl yh xl xh]
%
%Per-Erik Forssen, July 2005
function [P1,P2]=clip_lines2(L,imbox)

Nl=size(L,2); % Number of lines

% Lists of endpoints
P1=zeros(2,Nl);
P2=zeros(2,Nl);

e1=[0 1 -imbox(1)];  % Top side
e2=[0 1 -imbox(2)];  % Bottom side
e3=[1 0 -imbox(3)];  % Left side
e4=[1 0 -imbox(4)];  % Right side

L1=L(1,:);
L2=L(2,:);
L3=L(3,:);

% Intersect lines with top side
C1=L2*e1(3)-L3*e1(2);
%C2=L3*e1(1)-L1*e1(3);
C3=L1*e1(2)-L2*e1(1);

M1=double(C3==0);   % Intersection at infinity?
X1=C1./(C3+M1);     % X coord of intersection if exists
M1=(~M1).*(X1>=imbox(3)).*(X1<=imbox(4));

% Intersect lines with bottom side
C1=L2*e2(3)-L3*e2(2);
%C2=L3*e2(1)-L1*e2(3);
C3=L1*e2(2)-L2*e2(1);

M2=double(C3==0);   % Intersection at infinity?
X2=C1./(C3+M2);     % X coord of intersection if exists
M2=(~M2).*(X2>=imbox(3)).*(X2<=imbox(4));

% Intersect lines with left side
%C1=L2*e3(3)-L3*e3(2);
C2=L3*e3(1)-L1*e3(3);
C3=L1*e3(2)-L2*e3(1);

M3=double(C3==0);   % Intersection at infinity?
Y3=C2./(C3+M3);     % Y coord of intersection if exists
M3=(~M3).*(Y3>=imbox(1)).*(Y3<=imbox(2));

% Intersect lines with right side
%C1=L2*e4(3)-L3*e4(2);
C2=L3*e4(1)-L1*e4(3);
C3=L1*e4(2)-L2*e4(1);

M4=double(C3==0);   % Intersection at infinity?
Y4=C2./(C3+M4);     % Y coord of intersection if exists
M4=(~M4).*(Y4>=imbox(1)).*(Y4<=imbox(2));

% Find out which two points to use
MP=M1+2*M2+4*M3+8*M4;

% Pick one if several valid cases.
Msum=M1+M2+M3+M4;

% Value to subtract to remove first instance
MS1=M1+(~M1).*(2.*M2+(~M2).*(4.*M3+(~M3).*8.*M4));

% Value to subtract to remove second instance
MS2=M1.*M2*2+xor(M1,M2).*M3*4+xor(M1,xor(M2,M3)).*(~and(M1,and(M2,M3))).*M4*8;

MP=MP-(Msum==3).*MS1-(Msum==4).*MS2;

P1(1,:)=P1(1,:)+(MP==12).*imbox(3);
P1(2,:)=P1(2,:)+(MP==12).*Y3;
P2(1,:)=P2(1,:)+(MP==12).*imbox(4);
P2(2,:)=P2(2,:)+(MP==12).*Y4;

P1(1,:)=P1(1,:)+(MP==5).*imbox(3);
P1(2,:)=P1(2,:)+(MP==5).*Y3;
P2(1,:)=P2(1,:)+(MP==5).*X1;
P2(2,:)=P2(2,:)+(MP==5).*imbox(1);

P1(1,:)=P1(1,:)+(MP==6).*imbox(3);
P1(2,:)=P1(2,:)+(MP==6).*Y3;
P2(1,:)=P2(1,:)+(MP==6).*X2;
P2(2,:)=P2(2,:)+(MP==6).*imbox(2);

P1(1,:)=P1(1,:)+(MP==9).*imbox(4);
P1(2,:)=P1(2,:)+(MP==9).*Y4;
P2(1,:)=P2(1,:)+(MP==9).*X1;
P2(2,:)=P2(2,:)+(MP==9).*imbox(1);

P1(1,:)=P1(1,:)+(MP==10).*imbox(4);
P1(2,:)=P1(2,:)+(MP==10).*Y4;
P2(1,:)=P2(1,:)+(MP==10).*X2;
P2(2,:)=P2(2,:)+(MP==10).*imbox(2);

P1(1,:)=P1(1,:)+(MP==3).*X1;
P1(2,:)=P1(2,:)+(MP==3).*imbox(1);
P2(1,:)=P2(1,:)+(MP==3).*X2;
P2(2,:)=P2(2,:)+(MP==3).*imbox(2);

if(sum(MP==0)>0)
  fprintf('WARNING: %d lines fall outside image.\n',sum(MP==0));
end

