%function Im=draw_circle(Im0,p0,radius,lw,typefl)
%
% Draw a circle with sub-pixel precision
% in a grey-scale image.
%
% IM0    Image to draw in [rows cols]
% P0     Circle origin [x y]
% RADIUS Circle radius (pixels)
% LW     Line witdh (default 0.5)
% TYPEFL Select part of circle to draw
%          1 for lower half
%          2 for upper half
%          3 entire circle (default)
%
% See also DRAW_LINES.
%
%Per-Erik Forssen, May 2005

function Im=draw_circle(Im0,p0,radius,lw,typefl)

if nargin<4,lw=0.5;end
if nargin<5,typefl=3;end

N=30;   % Number of segments in half-circle

t=([1:N]-1)/(N-1)*pi;

P1=[];
P2=[];

Px=radius*cos(t);
Py=radius*sin(t);

if bitand(typefl,1),
  P1=[P1 [p0(1)+Px(1:end-1);p0(2)+Py(1:end-1)]];
  P2=[P2 [p0(1)+Px(2:end);p0(2)+Py(2:end)]];
end

if bitand(typefl,2),
  P1=[P1 [p0(1)+Px(1:end-1);p0(2)-Py(1:end-1)]];
  P2=[P2 [p0(1)+Px(2:end);p0(2)-Py(2:end)]];
end

% Draw the circle
Im=draw_lines(Im0,P1,P2,lw);
