%function Im=draw_ellipse(Im0,m,I,lw)
%
% Draw an ellipse with sub-pixel precision
% in a grey-scale image.
%
% IM0    Image to draw in [rows cols]
% M      Origin/Centre of gravity [x y]
% I      Inertia matrix .25*R'*D^2*R (2x2)
% LW     Line witdh (default 0.5)
%
% See also DRAW_LINES, DRAW_CIRCLE.
%
%Per-Erik Forssen, October 2005

function Im=draw_ellipse(Im0,m,I,lw)

if nargin<4,lw=0.5;end

N=60;   % Number of segments in ellipse

[E,D]=eig(I);    % I=E*D*E' => rot'=E or rot'=[0 1;1 0]*E
R=E*2*sqrtm(D);

t=([1:N]-1)/(N-1)*2*pi;
x=[cos(t);sin(t)];
Px=R(1,:)*x+m(1);
Py=R(2,:)*x+m(2);

P1=[Px(1:end-1);Py(1:end-1)];
P2=[Px(2:end);Py(2:end)];

% Draw the ellipse
Im=draw_lines(Im0,P1,P2,lw);
