/*
**  draw_lines.c
*/

#include <math.h>
#include "mex.h"
#define PI 3.141592653589793143328792211

void draw_lines(double *img0,double *img1,const int *dims,double *pointlist1,
		double *pointlist2,int npts,double width) {
int rows,cols,x,y,k,cind;
int xl,xh,yl,yh,cstate; 
double xp1,yp1,xp2,yp2,nval,d,ld;
double normal[2],tangent[2],lnorm,pp1,pp2,temp;

 rows=dims[0];
 cols=dims[1];
 /* First copy image */
 for(x=0;x<cols;x++) {
   for(y=0;y<rows;y++) {
     cind=x*rows+y;
     img1[cind]=img0[cind];
   }
 }
 /* Now paint points */
 for(k=0;k<npts;k++) {
   xp1=pointlist1[k*2]-1.0; /* Matlab compensation */
   yp1=pointlist1[k*2+1]-1.0;
   xp2=pointlist2[k*2]-1.0; /* Matlab compensation */
   yp2=pointlist2[k*2+1]-1.0;
   /* Compute tangent vector */
   tangent[0]=xp2-xp1;
   tangent[1]=yp2-yp1;
   lnorm=sqrt(tangent[0]*tangent[0]+tangent[1]*tangent[1]);
   tangent[0]/=lnorm;
   tangent[1]/=lnorm;
   /* Compute normal vector */
   normal[0]=-tangent[1];
   normal[1]=tangent[0];
   pp1=tangent[0]*xp1+tangent[1]*yp1; /* Project points onto tangent */
   pp2=tangent[0]*xp2+tangent[1]*yp2;
   /* Ensure pp1<pp2 */
   if(pp1>pp2) {temp=pp1;pp1=pp2;pp2=temp;} 
   
   /* Compute line bounding box */
   if(xp1<xp2) {
     xl=ceil(xp1-2*width);
     xh=floor(xp2+2*width);
   } else {
     xl=ceil(xp2-2*width);
     xh=floor(xp1+2*width);
   }
   if(xl<0) xl=0;
   if(xh<0) xh=0;
   if(xl>cols-1) xl=cols-1;
   if(xh>cols-1) xh=cols-1;
   if(yp1<yp2) {
     yl=ceil(yp1-2*width);
     yh=floor(yp2+2*width);
   } else {
     yl=ceil(yp2-2*width);
     yh=floor(yp1+2*width);
   }
   if(yl<0) yl=0;
   if(yh<0) yh=0;
   if(yl>rows-1) yl=rows-1;
   if(yh>rows-1) yh=rows-1;
   /* Loop over line bounding box */
   if(tangent[1]*tangent[0]>0) {
     /* Positive slope */
     for(x=xl;x<=xh;x++) {
       y=yl;cstate=2;   /* Start here */
       while(cstate) {
	 /*for(y=yl;y<=yh;y++) {*/       
	 cind=x*rows+y;
	 /* Compute position on line */
	 d=tangent[0]*x+tangent[1]*y;
	 ld=3.0; /* Not on line */
	 if((d>pp1)&&(d<pp2)) {
	   /* Point could be on line */
	   ld=fabs(normal[0]*(x-xp1)+normal[1]*(y-yp1))-width+1;
	 } else {
	   /* Check line line end 1 */
	   ld=sqrt((xp1-x)*(xp1-x)+(yp1-y)*(yp1-y))-width+1;
	   if(ld>=2) {
	     /* Check line line end 2 */
	     ld=sqrt((xp2-x)*(xp2-x)+(yp2-y)*(yp2-y))-width+1;
	   }
	   /* Don't update yl at line ends */
	   if(ld<2) if(cstate==2) cstate=1;
	 }
	 /* Apply line function (1+cos(f))/2 */
	 if(ld<2) {
	   if(ld<0) ld=0;
	   nval=(1+cos(PI/2*ld))/2;
	   if(nval>img1[cind]) img1[cind]=nval;
	   if(cstate==2) {yl=y;cstate=1;}
	 } else {
	   /* Not on line, but have we just been there? */
	   if(cstate==1) cstate=0;
	 }
	 if(++y>yh) cstate=0;
       }
     }
   } else {
     /* Negative slope */
     for(x=xl;x<=xh;x++) {
       y=yh;cstate=2;   /* Start here */
       while(cstate) {
	 /*for(y=yl;y<=yh;y++) {*/       
	 cind=x*rows+y;
	 /* Compute position on line */
	 d=tangent[0]*x+tangent[1]*y;
	 ld=3.0; /* Not on line */
	 if((d>pp1)&&(d<pp2)) {
	   /* Point could be on line */
	   ld=fabs(normal[0]*(x-xp1)+normal[1]*(y-yp1))-width+1;
	 } else {
	   /* Check line line end 1 */
	   ld=sqrt((xp1-x)*(xp1-x)+(yp1-y)*(yp1-y))-width+1;
	   if(ld>=2) {
	     /* Check line line end 2 */
	     ld=sqrt((xp2-x)*(xp2-x)+(yp2-y)*(yp2-y))-width+1;
	   }
	   /* Don't update yh at line ends */
	   if(ld<2) if(cstate==2) cstate=1;
	 }
	 /* Apply line function (1+cos(f))/2 */
	 if(ld<2) {
	   if(ld<0) ld=0;
	   nval=(1+cos(PI/2*ld))/2;
	   if(nval>img1[cind]) img1[cind]=nval;
	   if(cstate==2) {yh=y;cstate=1;}
	 } else {
	   /* Not on line, but have we just been there? */
	   if(cstate==1) cstate=0;
	 }
	 if(--y<yl) cstate=0;
       }
     }
   }
 }
}
/*
** Function to check wether an argument is
** a double 2d matrix of given size
** -1 as size means any size
*/
int has_size(const mxArray *arg,int mdims,int *msize)
{
const int *dims;
int k,retval;
  dims = mxGetDimensions(arg);
  retval=1;
  if(mxGetClassID(arg) != mxDOUBLE_CLASS) retval=0;
  if(mxGetNumberOfDimensions(arg) != mdims) retval=0;
  for(k=0;k<mdims;k++) {
    if((msize[k] != -1) && (dims[k] != msize[k])) retval=0;
  }
  return retval;
}
/*
**  mexFunction is the interface function to MATLAB.
**
** plhs - pointer to left-hand OUTPUT mxArrays
** nlhs - number of left-hand OUTPUT arrays
** prhs - pointer to right-hand INPUT mxArrays
** nrhs - number of right-hand INPUT arrays
**
*/
void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
const int *img_dims,*arg_dims;
int arg_ndims;
int out_dims[]={0,0,0};
double *img0,*img1,*pointlist1,*pointlist2;
double width=1.0;  /* Default value */
int npts;
/* Argument count check */
 if((nrhs <3)||(nrhs>4)) {
   if(nrhs !=0) mexPrintf("Error: 3-4 input arguments expected.\n");
   goto mexErrExit;
 }
    
/* Parse arg 1 (image) */
 img_dims = mxGetDimensions(prhs[0]);
 arg_ndims = mxGetNumberOfDimensions(prhs[0]);
 if(mxGetClassID(prhs[0]) != mxDOUBLE_CLASS) {
   mexPrintf("Type mismatch: <Im0> should be of class double.\n");
   goto mexErrExit;
 }
 if(arg_ndims !=2) {
   mexPrintf("Type mismatch: <Im0> should be 2D.\n");
   goto mexErrExit;
 }
/* Parse arg 2 (point list1) */
 arg_dims = mxGetDimensions(prhs[1]);
 arg_ndims = mxGetNumberOfDimensions(prhs[1]);
 if(mxGetClassID(prhs[1]) != mxDOUBLE_CLASS) {
   mexPrintf("Type mismatch: <Pl1> should be of class double.\n");
   goto mexErrExit;
 }
 if(arg_ndims !=2) {
   mexPrintf("Type mismatch: <Pl1> should be 2D.\n");
   goto mexErrExit;
 }
 if(arg_dims[0]!=2) {
   mexPrintf("Type mismatch: <Pl1> should be 2xN.\n");
   goto mexErrExit;
 }
 pointlist1=mxGetPr(prhs[1]);
 npts=arg_dims[1];

/* Parse arg 3 (point list2) */
 arg_dims = mxGetDimensions(prhs[2]);
 arg_ndims = mxGetNumberOfDimensions(prhs[2]);
 if(mxGetClassID(prhs[2]) != mxDOUBLE_CLASS) {
   mexPrintf("Type mismatch: <Pl2> should be of class double.\n");
   goto mexErrExit;
 }
 if(arg_ndims !=2) {
   mexPrintf("Type mismatch: <Pl2> should be 2D.\n");
   goto mexErrExit;
 }
 if(arg_dims[0]!=2) {
   mexPrintf("Type mismatch: <Pl2> should be 2xN.\n");
   goto mexErrExit;
 }
 if(arg_dims[1]!=npts) {
   mexPrintf("Error: <Pl1> and <Pl2> have different sizes.\n");
   goto mexErrExit;
 }
 pointlist2=mxGetPr(prhs[2]);

/* Parse arg 4 (width) */
 if(nrhs>3) {
   arg_dims = mxGetDimensions(prhs[3]);
   arg_ndims = mxGetNumberOfDimensions(prhs[3]);
   if(mxGetClassID(prhs[3]) != mxDOUBLE_CLASS) {
     mexPrintf("Type mismatch: <width> should be of class double.\n");
     goto mexErrExit;
   }
   if((arg_dims[0]!=arg_dims[1])||(arg_dims[1]!=1)) {
     mexPrintf("Type mismatch: <width> should be a scalar.\n");
     goto mexErrExit;
   }
   width=mxGetScalar(prhs[3]);
 }
/* Call visualisation functions */
 if(nlhs>0) {
   /* Create an empty image */
   plhs[0] = mxCreateNumericArray(2,img_dims,mxDOUBLE_CLASS,mxREAL);
   /* plot lines in image */
   draw_lines(mxGetPr(prhs[0]),mxGetPr(plhs[0]),img_dims,pointlist1,pointlist2,npts,width);
 }
/* Jump to normal exit */
 goto mexExit;

/* Exit with error message */
mexErrExit:
 mexPrintf("Usage: <Im>=draw_lines(<Im0>,<Pl1>,<Pl2>[,<width>])\n");
mexExit: {}
}

