%function Im=draw_marker(Im0,ep,lw,mtype,sz)
%
% Draw a markers at positions EP in image Im0,
% using DRAW_LINES.
%
% Im0   Image to draw in
% EP    Marker centre list [x;y]
% LW    Line width (default 1)
% MTYPE Marker type: 'x','+','*','^','s' or 'o' (default 'x')
% SZ    Marker diameter (default 15)
%
% See also DRAW_LINES.
%
%Per-Erik Forssen, April 2004
function Im=draw_marker(Im0,ep,lw,mtype,sz)

if nargin<3,lw=1;end
if nargin<4,mtype='x';end
if nargin<5,sz=15;end

Np=size(ep,2);

P1=[];P2=[];

switch(mtype),
 case 'x',
  for k=1:Np
    P1=[P1 [ep(1,k)+sz*[-1 -1]/sqrt(2);ep(2,k)+sz*[-1 1]/sqrt(2)]];
    P2=[P2 [ep(1,k)+sz*[1 1]/sqrt(2);ep(2,k)+sz*[1 -1]/sqrt(2)]];
  end
 case '+',
  for k=1:Np
    P1=[P1 [ep(1,k)+sz*[1 0];ep(2,k)+sz*[0 1]]];
    P2=[P2 [ep(1,k)+sz*[-1 0];ep(2,k)+sz*[0 -1]]];
  end
 case '*',
  s2=sqrt(2);
  for k=1:Np,
    P1=[P1 [ep(1,k)+sz*[-1 -1 s2 0]/s2;ep(2,k)+sz*[-1 1 0  s2]/s2]];
    P2=[P2 [ep(1,k)+sz*[1  1 -s2 0]/s2;ep(2,k)+sz*[1 -1 0 -s2]/s2]];
  end
 case '^',
  s3=1/sqrt(3);
  for k=1:Np,
    P1=[P1 [ep(1,k)+sz*[-1 1 0];ep(2,k)+sz*[s3 s3 -2*s3]]];
    P2=[P2 [ep(1,k)+sz*[1 0 -1];ep(2,k)+sz*[s3 -2*s3 s3]]];
  end
 case 's',
  for k=1:Np,
    P1=[P1 [ep(1,k)+sz*[-1 1 1 -1];ep(2,k)+sz*[-1 -1 1 1]]];
    P2=[P2 [ep(1,k)+sz*[1 1 -1 -1];ep(2,k)+sz*[-1 1 1 -1]]];
  end
 case 'o',
  t=[0:pi/10:2*pi];
  for k=1:Np,
    P1=[P1 [ep(1,k)+sz*cos(t(1:end-1));ep(2,k)+sz*sin(t(1:end-1))]];
    P2=[P2 [ep(1,k)+sz*cos(t(2:end));ep(2,k)+sz*sin(t(2:end))]];
  end
 otherwise
  error(['Marker type "' mtype '" not implemented.']);
end
    
Im=draw_lines(Im0,P1,P2,lw);
