//hbackground.h
//function protoypes and definitions for the homogeneous background detector

//Copyright (C) 2007 Matthew Baumann

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

#ifndef HBACKGROUND
#define HBACKGROUND

//---- includes -----------------------------------------------------------

//general
#include <cstdio>
#include <cstdlib>

//segmentation
#include <image.h>
#include <misc.h>
#include <pnmfile.h>
#include "segment-image.h"

//opencv
#include <cv.h>
#include <highgui.h>

//---- constants ----------------------------------------------------------

//default parameter values:

//segmentation parameters
#define SIGMA 3					//pre-segmentation smoothing
#define K 100					//threshold setting
#define MIN_SIZE 20				//minimum segment size

//threshold parameters
//the pixel ratio threshold was chosen because the ratio of the area of circle 
//inscribed in a square to the square is pi*r^2 / 4*r^2 = pi/4 ~= 0.78981
//so the area of the surrounding background is ~= 0.21018
#define PIXEL_THRESHOLD 0.2;	//ratio of total pixels (< 1 - pi/4)
#define BORDER_THRESHOLD 0.4;  //ratio of border pixels (arbitrary) 

//---- functions ----------------------------------------------------------

//homogeneous background test
//PRE: takes an openCV IplImage and the five parameters (see above)
//POST: return true of the image has an homogeneous background.
//      returns false otherwise
//		the image reference &output receives the segmented result image (for debugging or output)
bool hbackground(const IplImage* img, float sigma, float k, int min_size, double pixel_threshold, double border_threshold, segment::image<segment::rgb>* &output)
{
	int height,width,step,channels;
	uchar *data;
	
	// get the image data
	height    = img->height;
	width     = img->width;
	step      = img->widthStep;
	channels  = img->nChannels;
	data      = (uchar *)img->imageData;
	
	//convert to segment::image<segment::rgb>
	segment::image<segment::rgb> *input = new segment::image<segment::rgb>(width,height,true);
  
	for(int i=0;i<height;i++)
	{
		for(int j=0;j<width;j++) 
		{
			input->access[i][j].r =255-data[i*step+j*channels+0];  //r
			input->access[i][j].g =255-data[i*step+j*channels+1];  //g
			input->access[i][j].b =255-data[i*step+j*channels+2];  //b
    	}
  	}
	
	int num_ccs;    //number of connected components
	
	//call the hbackground routine
	bool result = segment::hbackground(input, sigma, k, min_size, &num_ccs, pixel_threshold, border_threshold, output); 
	
	if(output == NULL)
	{
		printf("output is NULL\n");
	}
	
	return result;
}


#endif
