//this is a command-line UI for the homogeneous background detector
//this is intended as an example of how to use the detector

//Copyright (C) 2007 Matthew Baumann

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA


#include "hbackground.h"

int main(int argc, char* argv[]) {
  //grab the file name	
  if (argc != 2 && argc != 7 && argc != 8) {
    fprintf(stderr, "usage: %s <input file name> [<sigma> <k> <min size> <pixel threshold> <border threshold> [output image name]]\n", argv[0]);
    return 1;
  }
  
  //set defaults
	float sigma = SIGMA;
	float k = K;
	int min_size = MIN_SIZE;
	
	double pixel_threshold = PIXEL_THRESHOLD;
	double border_threshold = BORDER_THRESHOLD;
  
  
  if(argc == 7 || argc == 8)
  {
  	//if parameters given, load them
  	sigma = atof(argv[2]);
	k = atof(argv[3]);
	min_size = atoi(argv[4]);
	
	pixel_threshold = atof(argv[5]);
	border_threshold = atof(argv[6]);
  }
  
  printf("%d arguments. sigma=%f k=%f min_size=%d pthresh=%f bthresh=%f\n",argc,sigma,k,min_size,pixel_threshold,border_threshold);
  
  //create the openCV (Ipl) image and metadata variables
  IplImage* img = 0; 
	
  //load openCV image
  //printf("loading input image %s.\n",argv[1]); 
  img=cvLoadImage(argv[1]);
  if(!img){
    printf("ERROR: Could not load image file: %s\n",argv[1]);
    exit(0);
  }
  
  //create storage for the segmented output image
  segment::image<segment::rgb>* output = NULL;
  
  //test if the image has an homogeneous background
  //also fills in the output image with the segmented map
  bool homogeneous = hbackground(img,sigma,k,min_size,pixel_threshold,border_threshold, output);
  
  //if requested, output the segmented image
  if(output != NULL && argc == 8)
  {
    segment::savePPM(output,argv[7]);
    printf("Wrote image file %s\n",argv[7]);
  }
  
  return homogeneous;

}
