#!/bin/bash

USER_AGENT="Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en-US; rv:1.8.0.7) Gecko/20060909 Firefox/1.5.0.7"

#check for the correct number of arguments
#if it's not 2, then exit with error code and print a usage string
if [ $# -ne 3 ]
then 
    echo "Usage: `basename $0` <list filename> <directory name> <num_images>"
    exit 64
fi

FILENAME=$1
URL_LIST=`cat $FILENAME`

DIRECTORY=$2

MAX_NUM=$3

mkdir $DIRECTORY

#echo $URL_LIST

COUNTER=0
#while [ $COUNTER -le $MAX_NUM ]; do

	#download all the images in the Image list
#	wget "-nv" "-T" "10" "-U ""$USER_AGENT" "-P""$DIRECTORY" $URL_LIST[COUNTER]
#	COUNTER++

#done

for URL in $URL_LIST
do
  if [ $COUNTER -ge $MAX_NUM ] 
      then
      exit 0
  fi

  ANTICIPATEDNAME=${URL##*}
  ANTICIPATEDEXTENSION=${URL##*.}
	# stderr redirected to stdout to get output from wget
  WGETRESULT=`wget "-nv" "-T" "10" "--tries=1" "-U ""$USER_AGENT" "-P""$DIRECTORY" $URL 2>&1`
  if [ $? -eq 0 ]; then #if wget returned 0 (all good)
      
      echo "wget result: $WGETRESULT";
      
      FIRSTQUOTE=`expr index "$WGETRESULT" "\""`;
      CHOPPED=${WGETRESULT:$FIRSTQUOTE};
      let NEXTQUOTE=`expr index "$CHOPPED" "\""`-1;
      CHOPPED=${CHOPPED:0:$NEXTQUOTE};
      SLASHINDEX=`expr index "$CHOPPED" "/"`;
      RESULTNAME=${CHOPPED:$SLASHINDEX};
      
      NAME=${RESULTNAME##*/}
      EXTENSION=${RESULTNAME##*.}

      IMAGECHECK=`identify -format "%i" /$RESULTNAME 2>&1`; # will return the name if file is actually an image
      if [ "/$RESULTNAME" == "$IMAGECHECK" ] && [ $? -eq 0 ]; then
	  mv "/$RESULTNAME" "$DIRECTORY/$COUNTER.$EXTENSION";
	  
		# conditionally resize image so it has a maximum width of 500
		# (doesn't actually make the image a 500x500 square)
		# also convert to png since we're already re-sizing
	  convert "-resize" "500x500>" "-quality" "95%" "$DIRECTORY/$COUNTER.$EXTENSION" "$DIRECTORY/$COUNTER.jpg";
		# conditionallly resize image so it has a minimum dimension of 200 on any side...
		# workaround to bug in keypoints extractor
	  convert "-resize" "200x200<" "-quality" "95%" "$DIRECTORY/$COUNTER.jpg" "$DIRECTORY/$COUNTER.jpg";
	  
	        # make sure this isn't a wierd single channel faked rgb
	  CLASSCHECK=`identify -format "%r" $DIRECTORY/$COUNTER.jpg`;
	  if [ "$CLASSCHECK" != "PseudoClassRGB" ]; then
	            # dete original image (unless it was a png and just got overwritten)... keeping only resized png
	      if [ "$EXTENSION" != "jpg" ]; then
		  rm -f "$DIRECTORY/$COUNTER.$EXTENSION";
	      fi
	      let COUNTER=COUNTER+1 # only incremented when wget succeeds and it was an image that was downloaded
	  else
	      rm -f "$DIRECTORY/$COUNTER.$EXTENSION";
	      rm -f "$DIRECTORY/$COUNTER.jpg";
	  fi
	  
      else
	  echo "/$RESULTNAME is not an image. Deleting...";
	  rm -f "/$RESULTNAME";
      fi
      
  else
      echo "wget failed... assuming the image $URL was not fetched";
      # try to delete just in case
      rm -f "$DIRECTORY/$ANTICIPATEDNAME.$ANTICIPATEDEXTENSION";
  fi

  #One way to get rid of long files that weren't parsed correctly somewhere along the line
  #like filenames with french characters in the name
  command="rm -fv $DIRECTORY/???*.*"
  $command;
done
